#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This modules contains objects that represents Telegram Replies"""

from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._chat import Chat
from telegram._checklists import Checklist
from telegram._dice import Dice
from telegram._files.animation import Animation
from telegram._files.audio import Audio
from telegram._files.contact import Contact
from telegram._files.document import Document
from telegram._files.location import Location
from telegram._files.photosize import PhotoSize
from telegram._files.sticker import Sticker
from telegram._files.venue import Venue
from telegram._files.video import Video
from telegram._files.videonote import VideoNote
from telegram._files.voice import Voice
from telegram._games.game import Game
from telegram._giveaway import Giveaway, GiveawayWinners
from telegram._linkpreviewoptions import LinkPreviewOptions
from telegram._messageentity import MessageEntity
from telegram._messageorigin import MessageOrigin
from telegram._paidmedia import PaidMediaInfo
from telegram._payment.invoice import Invoice
from telegram._poll import Poll
from telegram._story import Story
from telegram._telegramobject import TelegramObject
from telegram._utils.argumentparsing import de_json_optional, de_list_optional, parse_sequence_arg
from telegram._utils.defaultvalue import DEFAULT_NONE
from telegram._utils.types import JSONDict, ODVInput

if TYPE_CHECKING:
    from telegram import Bot


class ExternalReplyInfo(TelegramObject):
    """
    This object contains information about a message that is being replied to, which may
    come from another chat or forum topic.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`origin` is equal.

    .. versionadded:: 20.8

    Args:
        origin (:class:`telegram.MessageOrigin`): Origin of the message replied to by the given
            message.
        chat (:class:`telegram.Chat`, optional): Chat the original message belongs to. Available
            only if the chat is a supergroup or a channel.
        message_id (:obj:`int`, optional): Unique message identifier inside the original chat.
            Available only if the original chat is a supergroup or a channel.
        link_preview_options (:class:`telegram.LinkPreviewOptions`, optional): Options used for
            link preview generation for the original message, if it is a text message
        animation (:class:`telegram.Animation`, optional): Message is an animation, information
            about the animation.
        audio (:class:`telegram.Audio`, optional): Message is an audio file, information about the
            file.
        document (:class:`telegram.Document`, optional): Message is a general file, information
            about the file.
        photo (Sequence[:class:`telegram.PhotoSize`], optional): Message is a photo, available
            sizes of the photo.
        sticker (:class:`telegram.Sticker`, optional): Message is a sticker, information about the
            sticker.
        story (:class:`telegram.Story`, optional): Message is a forwarded story.
        video (:class:`telegram.Video`, optional): Message is a video, information about the video.
        video_note (:class:`telegram.VideoNote`, optional): Message is a `video note
            <https://telegram.org/blog/video-messages-and-telescope>`_, information about the video
            message.
        voice (:class:`telegram.Voice`, optional): Message is a voice message, information about
            the file.
        has_media_spoiler (:obj:`bool`, optional): :obj:`True`, if the message media is covered by
            a spoiler animation.
        checklist (:class:`telegram.Checklist`, optional): Message is a checklist.

            .. versionadded:: 22.3
        contact (:class:`telegram.Contact`, optional): Message is a shared contact, information
            about the contact.
        dice (:class:`telegram.Dice`, optional): Message is a dice with random value.
        game (:Class:`telegram.Game`. optional): Message is a game, information about the game.
            :ref:`More about games >> <games-tree>`.
        giveaway (:class:`telegram.Giveaway`, optional): Message is a scheduled giveaway,
            information about the giveaway.
        giveaway_winners (:class:`telegram.GiveawayWinners`, optional): A giveaway with public
            winners was completed.
        invoice (:class:`telegram.Invoice`, optional): Message is an invoice for a payment,
            information about the invoice. :ref:`More about payments >> <payments-tree>`.
        location (:class:`telegram.Location`, optional): Message is a shared location, information
            about the location.
        poll (:class:`telegram.Poll`, optional): Message is a native poll, information about the
            poll.
        venue (:class:`telegram.Venue`, optional): Message is a venue, information about the venue.
        paid_media (:class:`telegram.PaidMedia`, optional): Message contains paid media;
            information about the paid media.

            .. versionadded:: 21.4

    Attributes:
        origin (:class:`telegram.MessageOrigin`): Origin of the message replied to by the given
            message.
        chat (:class:`telegram.Chat`): Optional. Chat the original message belongs to. Available
            only if the chat is a supergroup or a channel.
        message_id (:obj:`int`): Optional. Unique message identifier inside the original chat.
            Available only if the original chat is a supergroup or a channel.
        link_preview_options (:class:`telegram.LinkPreviewOptions`): Optional. Options used for
            link preview generation for the original message, if it is a text message.
        animation (:class:`telegram.Animation`): Optional. Message is an animation, information
            about the animation.
        audio (:class:`telegram.Audio`): Optional. Message is an audio file, information about the
            file.
        document (:class:`telegram.Document`): Optional. Message is a general file, information
            about the file.
        photo (tuple[:class:`telegram.PhotoSize`]): Optional. Message is a photo, available sizes
            of the photo.
        sticker (:class:`telegram.Sticker`): Optional. Message is a sticker, information about the
            sticker.
        story (:class:`telegram.Story`): Optional. Message is a forwarded story.
        video (:class:`telegram.Video`): Optional. Message is a video, information about the video.
        video_note (:class:`telegram.VideoNote`): Optional. Message is a `video note
            <https://telegram.org/blog/video-messages-and-telescope>`_, information about the video
            message.
        voice (:class:`telegram.Voice`): Optional. Message is a voice message, information about
            the file.
        has_media_spoiler (:obj:`bool`): Optional. :obj:`True`, if the message media is covered by
            a spoiler animation.
        checklist (:class:`telegram.Checklist`): Optional. Message is a checklist.

            .. versionadded:: 22.3
        contact (:class:`telegram.Contact`): Optional. Message is a shared contact, information
            about the contact.
        dice (:class:`telegram.Dice`): Optional. Message is a dice with random value.
        game (:Class:`telegram.Game`): Optional. Message is a game, information about the game.
            :ref:`More about games >> <games-tree>`.
        giveaway (:class:`telegram.Giveaway`): Optional. Message is a scheduled giveaway,
            information about the giveaway.
        giveaway_winners (:class:`telegram.GiveawayWinners`): Optional. A giveaway with public
            winners was completed.
        invoice (:class:`telegram.Invoice`): Optional. Message is an invoice for a payment,
            information about the invoice. :ref:`More about payments >> <payments-tree>`.
        location (:class:`telegram.Location`): Optional. Message is a shared location, information
            about the location.
        poll (:class:`telegram.Poll`): Optional. Message is a native poll, information about the
            poll.
        venue (:class:`telegram.Venue`): Optional. Message is a venue, information about the venue.
        paid_media (:class:`telegram.PaidMedia`): Optional. Message contains paid media;
            information about the paid media.

            .. versionadded:: 21.4
    """

    __slots__ = (
        "animation",
        "audio",
        "chat",
        "checklist",
        "contact",
        "dice",
        "document",
        "game",
        "giveaway",
        "giveaway_winners",
        "has_media_spoiler",
        "invoice",
        "link_preview_options",
        "location",
        "message_id",
        "origin",
        "paid_media",
        "photo",
        "poll",
        "sticker",
        "story",
        "venue",
        "video",
        "video_note",
        "voice",
    )

    def __init__(
        self,
        origin: MessageOrigin,
        chat: Chat | None = None,
        message_id: int | None = None,
        link_preview_options: LinkPreviewOptions | None = None,
        animation: Animation | None = None,
        audio: Audio | None = None,
        document: Document | None = None,
        photo: Sequence[PhotoSize] | None = None,
        sticker: Sticker | None = None,
        story: Story | None = None,
        video: Video | None = None,
        video_note: VideoNote | None = None,
        voice: Voice | None = None,
        has_media_spoiler: bool | None = None,
        contact: "Contact | None" = None,
        dice: Dice | None = None,
        game: Game | None = None,
        giveaway: Giveaway | None = None,
        giveaway_winners: GiveawayWinners | None = None,
        invoice: Invoice | None = None,
        location: "Location | None" = None,
        poll: Poll | None = None,
        venue: Venue | None = None,
        paid_media: PaidMediaInfo | None = None,
        checklist: Checklist | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        self.origin: MessageOrigin = origin
        self.chat: Chat | None = chat
        self.message_id: int | None = message_id
        self.link_preview_options: LinkPreviewOptions | None = link_preview_options
        self.animation: Animation | None = animation
        self.audio: Audio | None = audio
        self.document: Document | None = document
        self.photo: tuple[PhotoSize, ...] | None = parse_sequence_arg(photo)
        self.sticker: Sticker | None = sticker
        self.story: Story | None = story
        self.video: Video | None = video
        self.video_note: VideoNote | None = video_note
        self.voice: Voice | None = voice
        self.has_media_spoiler: bool | None = has_media_spoiler
        self.checklist: Checklist | None = checklist
        self.contact: Contact | None = contact
        self.dice: Dice | None = dice
        self.game: Game | None = game
        self.giveaway: Giveaway | None = giveaway
        self.giveaway_winners: GiveawayWinners | None = giveaway_winners
        self.invoice: Invoice | None = invoice
        self.location: Location | None = location
        self.poll: Poll | None = poll
        self.venue: Venue | None = venue
        self.paid_media: PaidMediaInfo | None = paid_media

        self._id_attrs = (self.origin,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "ExternalReplyInfo":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["origin"] = de_json_optional(data.get("origin"), MessageOrigin, bot)
        data["chat"] = de_json_optional(data.get("chat"), Chat, bot)
        data["link_preview_options"] = de_json_optional(
            data.get("link_preview_options"), LinkPreviewOptions, bot
        )
        data["animation"] = de_json_optional(data.get("animation"), Animation, bot)
        data["audio"] = de_json_optional(data.get("audio"), Audio, bot)
        data["document"] = de_json_optional(data.get("document"), Document, bot)
        data["photo"] = de_list_optional(data.get("photo"), PhotoSize, bot)
        data["sticker"] = de_json_optional(data.get("sticker"), Sticker, bot)
        data["story"] = de_json_optional(data.get("story"), Story, bot)
        data["video"] = de_json_optional(data.get("video"), Video, bot)
        data["video_note"] = de_json_optional(data.get("video_note"), VideoNote, bot)
        data["voice"] = de_json_optional(data.get("voice"), Voice, bot)
        data["contact"] = de_json_optional(data.get("contact"), Contact, bot)
        data["dice"] = de_json_optional(data.get("dice"), Dice, bot)
        data["game"] = de_json_optional(data.get("game"), Game, bot)
        data["giveaway"] = de_json_optional(data.get("giveaway"), Giveaway, bot)
        data["giveaway_winners"] = de_json_optional(
            data.get("giveaway_winners"), GiveawayWinners, bot
        )
        data["invoice"] = de_json_optional(data.get("invoice"), Invoice, bot)
        data["location"] = de_json_optional(data.get("location"), Location, bot)
        data["poll"] = de_json_optional(data.get("poll"), Poll, bot)
        data["venue"] = de_json_optional(data.get("venue"), Venue, bot)
        data["paid_media"] = de_json_optional(data.get("paid_media"), PaidMediaInfo, bot)
        data["checklist"] = de_json_optional(data.get("checklist"), Checklist, bot)

        return super().de_json(data=data, bot=bot)


class TextQuote(TelegramObject):
    """
    This object contains information about the quoted part of a message that is replied to
    by the given message.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`text` and :attr:`position` are equal.

    .. versionadded:: 20.8

    Args:
        text (:obj:`str`): Text of the quoted part of a message that is replied to by the given
            message.
        position (:obj:`int`): Approximate quote position in the original message in UTF-16 code
            units as specified by the sender.
        entities (Sequence[:class:`telegram.MessageEntity`], optional): Special entities that
            appear
            in the quote. Currently, only bold, italic, underline, strikethrough, spoiler, and
            custom_emoji entities are kept in quotes.
        is_manual (:obj:`bool`, optional): :obj:`True`, if the quote was chosen manually by the
            message sender. Otherwise, the quote was added automatically by the server.

    Attributes:
        text (:obj:`str`): Text of the quoted part of a message that is replied to by the given
            message.
        position (:obj:`int`): Approximate quote position in the original message in UTF-16 code
            units as specified by the sender.
        entities (tuple[:class:`telegram.MessageEntity`]): Optional. Special entities that appear
            in the quote. Currently, only bold, italic, underline, strikethrough, spoiler, and
            custom_emoji entities are kept in quotes.
        is_manual (:obj:`bool`): Optional. :obj:`True`, if the quote was chosen manually by the
            message sender. Otherwise, the quote was added automatically by the server.
    """

    __slots__ = (
        "entities",
        "is_manual",
        "position",
        "text",
    )

    def __init__(
        self,
        text: str,
        position: int,
        entities: Sequence[MessageEntity] | None = None,
        is_manual: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        self.text: str = text
        self.position: int = position
        self.entities: tuple[MessageEntity, ...] | None = parse_sequence_arg(entities)
        self.is_manual: bool | None = is_manual

        self._id_attrs = (
            self.text,
            self.position,
        )

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "TextQuote":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["entities"] = de_list_optional(data.get("entities"), MessageEntity, bot)

        return super().de_json(data=data, bot=bot)


class ReplyParameters(TelegramObject):
    """
    Describes reply parameters for the message that is being sent.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`message_id` is equal.

    .. versionadded:: 20.8

    .. versionchanged:: 22.5
        The :paramref:`checklist_task_id` parameter has been moved to the last position to
        maintain backward compatibility with versions prior to 22.4.
        This resolves a breaking change accidentally introduced in version 22.4. See the changelog
        for version 22.5 for more information.

    Args:
        message_id (:obj:`int`): Identifier of the message that will be replied to in the current
            chat, or in the chat :paramref:`chat_id` if it is specified.
        chat_id (:obj:`int` | :obj:`str`, optional): If the message to be replied to is from a
            different chat, |chat_id_channel|
            Not supported for messages sent on behalf of a business account and messages from
            channel direct messages chats.
        allow_sending_without_reply (:obj:`bool`, optional): |allow_sending_without_reply| Can be
            used only for replies in the same chat and forum topic.
        quote (:obj:`str`, optional): Quoted part of the message to be replied to; 0-1024
            characters after entities parsing. The quote must be an exact substring of the message
            to be replied to, including bold, italic, underline, strikethrough, spoiler, and
            custom_emoji entities. The message will fail to send if the quote isn't found in the
            original message.
        quote_parse_mode (:obj:`str`, optional): Mode for parsing entities in the quote. See
            :wiki:`formatting options <Code-snippets#message-formatting-bold-italic-code->` for
            more details.
        quote_entities (Sequence[:class:`telegram.MessageEntity`], optional): A JSON-serialized
            list
            of special entities that appear in the quote. It can be specified instead of
            :paramref:`quote_parse_mode`.
        quote_position (:obj:`int`, optional): Position of the quote in the original message in
            UTF-16 code units.
        checklist_task_id (:obj:`int`, optional): Identifier of the specific checklist task to be
            replied to.

            .. versionadded:: 22.4

    Attributes:
        message_id (:obj:`int`): Identifier of the message that will be replied to in the current
            chat, or in the chat :paramref:`chat_id` if it is specified.
        chat_id (:obj:`int` | :obj:`str`): Optional. If the message to be replied to is from a
            different chat, |chat_id_channel|
            Not supported for messages sent on behalf of a business account and messages from
            channel direct messages chats.
        allow_sending_without_reply (:obj:`bool`): Optional. |allow_sending_without_reply| Can be
            used only for replies in the same chat and forum topic.
        quote (:obj:`str`): Optional. Quoted part of the message to be replied to; 0-1024
            characters after entities parsing. The quote must be an exact substring of the message
            to be replied to, including bold, italic, underline, strikethrough, spoiler, and
            custom_emoji entities. The message will fail to send if the quote isn't found in the
            original message.
        quote_parse_mode (:obj:`str`): Optional. Mode for parsing entities in the quote. See
            :wiki:`formatting options <Code-snippets#message-formatting-bold-italic-code->` for
            more details.
        quote_entities (tuple[:class:`telegram.MessageEntity`]): Optional. A JSON-serialized list
            of special entities that appear in the quote. It can be specified instead of
            :paramref:`quote_parse_mode`.
        quote_position (:obj:`int`): Optional. Position of the quote in the original message in
            UTF-16 code units.
        checklist_task_id (:obj:`int`): Optional. Identifier of the specific checklist task to be
            replied to.

            .. versionadded:: 22.4
    """

    __slots__ = (
        "allow_sending_without_reply",
        "chat_id",
        "checklist_task_id",
        "message_id",
        "quote",
        "quote_entities",
        "quote_parse_mode",
        "quote_position",
    )

    def __init__(
        self,
        message_id: int,
        chat_id: int | str | None = None,
        allow_sending_without_reply: ODVInput[bool] = DEFAULT_NONE,
        quote: str | None = None,
        quote_parse_mode: ODVInput[str] = DEFAULT_NONE,
        quote_entities: Sequence[MessageEntity] | None = None,
        quote_position: int | None = None,
        checklist_task_id: int | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        self.message_id: int = message_id
        self.chat_id: int | str | None = chat_id
        self.allow_sending_without_reply: ODVInput[bool] = allow_sending_without_reply
        self.quote: str | None = quote
        self.quote_parse_mode: ODVInput[str] = quote_parse_mode
        self.quote_entities: tuple[MessageEntity, ...] | None = parse_sequence_arg(quote_entities)
        self.quote_position: int | None = quote_position
        self.checklist_task_id: int | None = checklist_task_id

        self._id_attrs = (self.message_id,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "ReplyParameters":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["quote_entities"] = tuple(
            de_list_optional(data.get("quote_entities"), MessageEntity, bot)
        )

        return super().de_json(data=data, bot=bot)



#! /usr/bin/env python
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains classes used for warnings issued by this library.

.. versionadded:: 20.0
"""

__all__ = ["PTBDeprecationWarning", "PTBRuntimeWarning", "PTBUserWarning"]


class PTBUserWarning(UserWarning):
    """
    Custom user warning class used for warnings in this library.

    .. seealso:: :wiki:`Exceptions, Warnings and Logging <Exceptions%2C-Warnings-and-Logging>`

    .. versionadded:: 20.0
    """

    __slots__ = ()


class PTBRuntimeWarning(PTBUserWarning, RuntimeWarning):
    """
    Custom runtime warning class used for warnings in this library.

    .. versionadded:: 20.0
    """

    __slots__ = ()


# https://www.python.org/dev/peps/pep-0565/ recommends using a custom warning class derived from
# DeprecationWarning. We also subclass from PTBUserWarning so users can easily 'switch off'
# warnings
class PTBDeprecationWarning(PTBUserWarning, DeprecationWarning):
    """
    Custom warning class for deprecations in this library.

    .. versionchanged:: 20.0
       Renamed TelegramDeprecationWarning to PTBDeprecationWarning.

    Args:
        version (:obj:`str`): The version in which the feature was deprecated.

            .. versionadded:: 21.2
        message (:obj:`str`): The message to display.

            .. versionadded:: 21.2

    Attributes:
        version (:obj:`str`): The version in which the feature was deprecated.

            .. versionadded:: 21.2
        message (:obj:`str`): The message to display.

            .. versionadded:: 21.2
    """

    __slots__ = ("message", "version")

    def __init__(self, version: str, message: str) -> None:
        self.version: str = version
        self.message: str = message

    def __str__(self) -> str:
        """Returns a string representation of the warning, using :attr:`message` and
        :attr:`version`.

        .. versionadded:: 21.2
        """
        return f"Deprecated since version {self.version}: {self.message}"



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import (
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    InlineQueryResultCachedVideo,
    InlineQueryResultCachedVoice,
    InputTextMessageContent,
    MessageEntity,
)
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def inline_query_result_cached_video():
    return InlineQueryResultCachedVideo(
        InlineQueryResultCachedVideoTestBase.id_,
        InlineQueryResultCachedVideoTestBase.video_file_id,
        InlineQueryResultCachedVideoTestBase.title,
        caption=InlineQueryResultCachedVideoTestBase.caption,
        parse_mode=InlineQueryResultCachedVideoTestBase.parse_mode,
        caption_entities=InlineQueryResultCachedVideoTestBase.caption_entities,
        description=InlineQueryResultCachedVideoTestBase.description,
        input_message_content=InlineQueryResultCachedVideoTestBase.input_message_content,
        reply_markup=InlineQueryResultCachedVideoTestBase.reply_markup,
        show_caption_above_media=InlineQueryResultCachedVideoTestBase.show_caption_above_media,
    )


class InlineQueryResultCachedVideoTestBase:
    id_ = "id"
    type_ = "video"
    video_file_id = "video file id"
    title = "title"
    caption = "caption"
    parse_mode = "Markdown"
    caption_entities = [MessageEntity(MessageEntity.ITALIC, 0, 7)]
    description = "description"
    input_message_content = InputTextMessageContent("input_message_content")
    reply_markup = InlineKeyboardMarkup([[InlineKeyboardButton("reply_markup")]])
    show_caption_above_media = True


class TestInlineQueryResultCachedVideoWithoutRequest(InlineQueryResultCachedVideoTestBase):
    def test_slot_behaviour(self, inline_query_result_cached_video):
        inst = inline_query_result_cached_video
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, inline_query_result_cached_video):
        assert inline_query_result_cached_video.type == self.type_
        assert inline_query_result_cached_video.id == self.id_
        assert inline_query_result_cached_video.video_file_id == self.video_file_id
        assert inline_query_result_cached_video.title == self.title
        assert inline_query_result_cached_video.description == self.description
        assert inline_query_result_cached_video.caption == self.caption
        assert inline_query_result_cached_video.parse_mode == self.parse_mode
        assert inline_query_result_cached_video.caption_entities == tuple(self.caption_entities)
        assert (
            inline_query_result_cached_video.input_message_content.to_dict()
            == self.input_message_content.to_dict()
        )
        assert (
            inline_query_result_cached_video.reply_markup.to_dict() == self.reply_markup.to_dict()
        )
        assert (
            inline_query_result_cached_video.show_caption_above_media
            == self.show_caption_above_media
        )

    def test_caption_entities_always_tuple(self):
        video = InlineQueryResultCachedVideo(self.id_, self.video_file_id, self.title)

        assert video.caption_entities == ()

    def test_to_dict(self, inline_query_result_cached_video):
        inline_query_result_cached_video_dict = inline_query_result_cached_video.to_dict()

        assert isinstance(inline_query_result_cached_video_dict, dict)
        assert (
            inline_query_result_cached_video_dict["type"] == inline_query_result_cached_video.type
        )
        assert inline_query_result_cached_video_dict["id"] == inline_query_result_cached_video.id
        assert (
            inline_query_result_cached_video_dict["video_file_id"]
            == inline_query_result_cached_video.video_file_id
        )
        assert (
            inline_query_result_cached_video_dict["title"]
            == inline_query_result_cached_video.title
        )
        assert (
            inline_query_result_cached_video_dict["description"]
            == inline_query_result_cached_video.description
        )
        assert (
            inline_query_result_cached_video_dict["caption"]
            == inline_query_result_cached_video.caption
        )
        assert (
            inline_query_result_cached_video_dict["parse_mode"]
            == inline_query_result_cached_video.parse_mode
        )
        assert inline_query_result_cached_video_dict["caption_entities"] == [
            ce.to_dict() for ce in inline_query_result_cached_video.caption_entities
        ]
        assert (
            inline_query_result_cached_video_dict["input_message_content"]
            == inline_query_result_cached_video.input_message_content.to_dict()
        )
        assert (
            inline_query_result_cached_video_dict["reply_markup"]
            == inline_query_result_cached_video.reply_markup.to_dict()
        )
        assert (
            inline_query_result_cached_video_dict["show_caption_above_media"]
            == inline_query_result_cached_video.show_caption_above_media
        )

    def test_equality(self):
        a = InlineQueryResultCachedVideo(self.id_, self.video_file_id, self.title)
        b = InlineQueryResultCachedVideo(self.id_, self.video_file_id, self.title)
        c = InlineQueryResultCachedVideo(self.id_, "", self.title)
        d = InlineQueryResultCachedVideo("", self.video_file_id, self.title)
        e = InlineQueryResultCachedVoice(self.id_, "", "")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains helper functions related to the std-lib asyncio module.

.. versionadded:: 21.11

Warning:
    Contents of this module are intended to be used internally by the library and *not* by the
    user. Changes to this module are not considered breaking changes and may not be documented in
    the changelog.
"""

import asyncio
from typing import Literal


class TrackedBoundedSemaphore(asyncio.BoundedSemaphore):
    """Simple subclass of :class:`asyncio.BoundedSemaphore` that tracks the current value of the
    semaphore. While there is an attribute ``_value`` in the superclass, it's private and we
    don't want to rely on it.
    """

    __slots__ = ("_current_value",)

    def __init__(self, value: int = 1) -> None:
        super().__init__(value)
        self._current_value = value

    @property
    def current_value(self) -> int:
        return self._current_value

    async def acquire(self) -> Literal[True]:
        await super().acquire()
        self._current_value -= 1
        return True

    def release(self) -> None:
        super().release()
        self._current_value += 1



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

"""
This file tests the case that PTB was installed *without* the optional dependency `passport`.
Currently this only means that cryptography is not installed.

Because imports in pytest are intricate, we just run
    pytest -k test_no_passport.py

with the TEST_WITH_OPT_DEPS environment variable set to False in addition to the regular test suite
"""

import pytest

import telegram
from telegram._passport import credentials
from tests.auxil.envvars import TEST_WITH_OPT_DEPS


@pytest.mark.skipif(
    TEST_WITH_OPT_DEPS, reason="Only relevant if the optional dependency is not installed"
)
class TestNoPassportWithoutRequest:
    def test_bot_init(self, bot_info):
        with pytest.raises(RuntimeError, match="passport"):
            telegram.Bot(bot_info["token"], private_key=1, private_key_password=2)

    def test_credentials_decrypt(self):
        with pytest.raises(RuntimeError, match="passport"):
            credentials.decrypt(1, 1, 1)

    def test_encrypted_credentials_decrypted_secret(self):
        ec = credentials.EncryptedCredentials("data", "hash", "secret")
        with pytest.raises(RuntimeError, match="passport"):
            ec.decrypted_secret



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
from telegram import Audio, UserProfileAudios
from tests.auxil.slots import mro_slots


class UserProfileAudiosTestBase:
    total_count = 2
    audios = [
        Audio("file_id1", "unique_id1", 512, "file_name1"),
        Audio("file_id2", "unique_id2", 512, "file_name2"),
    ]


class TestUserProfileAudiosWithoutRequest(UserProfileAudiosTestBase):
    def test_slot_behaviour(self):
        inst = UserProfileAudios(self.total_count, self.audios)
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {"total_count": 2, "audios": [x.to_dict() for x in self.audios]}
        user_profile_audios = UserProfileAudios.de_json(json_dict, offline_bot)
        assert user_profile_audios.api_kwargs == {}
        assert user_profile_audios.total_count == self.total_count
        assert user_profile_audios.audios == tuple(self.audios)

    def test_to_dict(self):
        user_profile_audios = UserProfileAudios(self.total_count, self.audios)
        user_profile_audios_dict = user_profile_audios.to_dict()
        assert user_profile_audios_dict["total_count"] == user_profile_audios.total_count
        for ix, x in enumerate(user_profile_audios_dict["audios"]):
            assert x == user_profile_audios.audios[ix].to_dict()

    def test_equality(self):
        a = UserProfileAudios(2, self.audios)
        b = UserProfileAudios(2, self.audios)
        c = UserProfileAudios(1, [self.audios[0]])
        d = Audio("file_id1", "unique_id1", 512, "file_name1")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



import re


def to_camel_case(snake_str):
    """https://stackoverflow.com/a/19053800"""
    components = snake_str.split("_")
    # We capitalize the first letter of each component except the first one
    # with the 'title' method and join them together.
    return components[0] + "".join(x.title() for x in components[1:])


def to_snake_case(camel_str):
    """https://stackoverflow.com/a/1176023"""
    name = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", camel_str)
    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", name).lower()



#!/usr/bin/env python
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import asyncio
import os
from io import BytesIO
from pathlib import Path

import pytest

from telegram import Bot, InputFile, MessageEntity, PhotoSize, ReplyParameters, Sticker
from telegram.constants import ParseMode
from telegram.error import BadRequest, TelegramError
from telegram.helpers import escape_markdown
from telegram.request import RequestData
from tests.auxil.bot_method_checks import (
    check_defaults_handling,
    check_shortcut_call,
    check_shortcut_signature,
)
from tests.auxil.build_messages import make_message
from tests.auxil.files import data_file
from tests.auxil.slots import mro_slots


class PhotoTestBase:
    width = 800
    height = 800
    caption = "<b>PhotoTest</b> - *Caption*"
    photo_file_url = "https://python-telegram-bot.org/static/testfiles/telegram_new.jpg"
    # For some reason the file size is not the same after switching to httpx
    # so we accept three different sizes here. Shouldn't be too much
    file_size = [29176, 27662, 27330]


class TestPhotoWithoutRequest(PhotoTestBase):
    def test_slot_behaviour(self, photo):
        for attr in photo.__slots__:
            assert getattr(photo, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(photo)) == len(set(mro_slots(photo))), "duplicate slot"

    def test_creation(self, thumb, photo):
        # Make sure file has been uploaded.
        assert isinstance(photo, PhotoSize)
        assert isinstance(photo.file_id, str)
        assert isinstance(photo.file_unique_id, str)
        assert photo.file_id
        assert photo.file_unique_id

        assert isinstance(thumb, PhotoSize)
        assert isinstance(thumb.file_id, str)
        assert isinstance(thumb.file_unique_id, str)
        assert thumb.file_id
        assert thumb.file_unique_id

    def test_expected_values(self, photo, thumb):
        assert photo.width == self.width
        assert photo.height == self.height
        assert photo.file_size in self.file_size
        assert thumb.width == 90
        assert thumb.height == 90
        # File sizes don't seem to be consistent, so we use the values that we have observed
        # so far
        assert thumb.file_size in [1475, 1477]

    def test_de_json(self, offline_bot, photo):
        json_dict = {
            "file_id": photo.file_id,
            "file_unique_id": photo.file_unique_id,
            "width": self.width,
            "height": self.height,
            "file_size": self.file_size,
        }
        json_photo = PhotoSize.de_json(json_dict, offline_bot)
        assert json_photo.api_kwargs == {}

        assert json_photo.file_id == photo.file_id
        assert json_photo.file_unique_id == photo.file_unique_id
        assert json_photo.width == self.width
        assert json_photo.height == self.height
        assert json_photo.file_size == self.file_size

    def test_to_dict(self, photo):
        photo_dict = photo.to_dict()

        assert isinstance(photo_dict, dict)
        assert photo_dict["file_id"] == photo.file_id
        assert photo_dict["file_unique_id"] == photo.file_unique_id
        assert photo_dict["width"] == photo.width
        assert photo_dict["height"] == photo.height
        assert photo_dict["file_size"] == photo.file_size

    def test_equality(self, photo):
        a = PhotoSize(photo.file_id, photo.file_unique_id, self.width, self.height)
        b = PhotoSize("", photo.file_unique_id, self.width, self.height)
        c = PhotoSize(photo.file_id, photo.file_unique_id, 0, 0)
        d = PhotoSize("", "", self.width, self.height)
        e = Sticker(
            photo.file_id,
            photo.file_unique_id,
            self.width,
            self.height,
            False,
            False,
            Sticker.REGULAR,
        )

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    async def test_error_without_required_args(self, offline_bot, chat_id):
        with pytest.raises(TypeError):
            await offline_bot.send_photo(chat_id=chat_id)

    async def test_send_photo_custom_filename(self, offline_bot, chat_id, photo_file, monkeypatch):
        async def make_assertion(url, request_data: RequestData, *args, **kwargs):
            return next(iter(request_data.multipart_data.values()))[0] == "custom_filename"

        monkeypatch.setattr(offline_bot.request, "post", make_assertion)
        assert await offline_bot.send_photo(chat_id, photo_file, filename="custom_filename")

    @pytest.mark.parametrize("local_mode", [True, False])
    async def test_send_photo_local_files(
        self, dummy_message_dict, monkeypatch, offline_bot, chat_id, local_mode
    ):
        try:
            offline_bot._local_mode = local_mode
            # For just test that the correct paths are passed as we have no local Bot API set up
            test_flag = False
            file = data_file("telegram.jpg")
            expected = file.as_uri()

            async def make_assertion(_, data, *args, **kwargs):
                nonlocal test_flag
                if local_mode:
                    test_flag = data.get("photo") == expected
                else:
                    test_flag = isinstance(data.get("photo"), InputFile)
                return dummy_message_dict

            monkeypatch.setattr(offline_bot, "_post", make_assertion)
            await offline_bot.send_photo(chat_id, file)
            assert test_flag
        finally:
            offline_bot._local_mode = False

    async def test_send_with_photosize(self, monkeypatch, offline_bot, chat_id, photo):
        async def make_assertion(url, request_data: RequestData, *args, **kwargs):
            return request_data.json_parameters["photo"] == photo.file_id

        monkeypatch.setattr(offline_bot.request, "post", make_assertion)
        assert await offline_bot.send_photo(photo=photo, chat_id=chat_id)

    async def test_get_file_instance_method(self, monkeypatch, photo):
        async def make_assertion(*_, **kwargs):
            return kwargs["file_id"] == photo.file_id

        assert check_shortcut_signature(PhotoSize.get_file, Bot.get_file, ["file_id"], [])
        assert await check_shortcut_call(photo.get_file, photo.get_bot(), "get_file")
        assert await check_defaults_handling(photo.get_file, photo.get_bot())

        monkeypatch.setattr(photo.get_bot(), "get_file", make_assertion)
        assert await photo.get_file()

    @pytest.mark.parametrize(
        ("default_bot", "custom"),
        [
            ({"parse_mode": ParseMode.HTML}, None),
            ({"parse_mode": ParseMode.HTML}, ParseMode.MARKDOWN_V2),
            ({"parse_mode": None}, ParseMode.MARKDOWN_V2),
        ],
        indirect=["default_bot"],
    )
    async def test_send_photo_default_quote_parse_mode(
        self, default_bot, chat_id, photo, custom, monkeypatch
    ):
        async def make_assertion(url, request_data: RequestData, *args, **kwargs):
            assert request_data.parameters["reply_parameters"].get("quote_parse_mode") == (
                custom or default_bot.defaults.quote_parse_mode
            )
            return make_message("dummy reply").to_dict()

        kwargs = {"message_id": 1}
        if custom is not None:
            kwargs["quote_parse_mode"] = custom

        monkeypatch.setattr(default_bot.request, "post", make_assertion)
        await default_bot.send_photo(chat_id, photo, reply_parameters=ReplyParameters(**kwargs))


class TestPhotoWithRequest(PhotoTestBase):
    async def test_send_photo_all_args(self, bot, chat_id, photo_file):
        message = await bot.send_photo(
            chat_id,
            photo_file,
            caption=self.caption,
            disable_notification=False,
            protect_content=True,
            parse_mode="Markdown",
            has_spoiler=True,
            show_caption_above_media=True,
        )

        assert isinstance(message.photo[-2], PhotoSize)
        assert isinstance(message.photo[-2].file_id, str)
        assert isinstance(message.photo[-2].file_unique_id, str)
        assert message.photo[-2].file_id
        assert message.photo[-2].file_unique_id

        assert isinstance(message.photo[-1], PhotoSize)
        assert isinstance(message.photo[-1].file_id, str)
        assert isinstance(message.photo[-1].file_unique_id, str)
        assert message.photo[-1].file_id
        assert message.photo[-1].file_unique_id

        assert message.caption == self.caption.replace("*", "")
        assert message.has_protected_content
        assert message.has_media_spoiler
        assert message.show_caption_above_media

    async def test_send_photo_parse_mode_markdown(self, bot, chat_id, photo_file):
        message = await bot.send_photo(
            chat_id, photo_file, caption=self.caption, parse_mode="Markdown"
        )
        assert isinstance(message.photo[-2], PhotoSize)
        assert isinstance(message.photo[-2].file_id, str)
        assert isinstance(message.photo[-2].file_unique_id, str)
        assert message.photo[-2].file_id
        assert message.photo[-2].file_unique_id

        assert isinstance(message.photo[-1], PhotoSize)
        assert isinstance(message.photo[-1].file_id, str)
        assert isinstance(message.photo[-1].file_unique_id, str)
        assert message.photo[-1].file_id
        assert message.photo[-1].file_unique_id

        assert message.caption == self.caption.replace("*", "")
        assert len(message.caption_entities) == 1

    async def test_send_photo_parse_mode_html(self, bot, chat_id, photo_file):
        message = await bot.send_photo(
            chat_id, photo_file, caption=self.caption, parse_mode="HTML"
        )
        assert isinstance(message.photo[-2], PhotoSize)
        assert isinstance(message.photo[-2].file_id, str)
        assert isinstance(message.photo[-2].file_unique_id, str)
        assert message.photo[-2].file_id
        assert message.photo[-2].file_unique_id

        assert isinstance(message.photo[-1], PhotoSize)
        assert isinstance(message.photo[-1].file_id, str)
        assert isinstance(message.photo[-1].file_unique_id, str)
        assert message.photo[-1].file_id
        assert message.photo[-1].file_unique_id

        assert message.caption == self.caption.replace("<b>", "").replace("</b>", "")
        assert len(message.caption_entities) == 1

    async def test_send_photo_caption_entities(self, bot, chat_id, photo_file):
        test_string = "Italic Bold Code"
        entities = [
            MessageEntity(MessageEntity.ITALIC, 0, 6),
            MessageEntity(MessageEntity.ITALIC, 7, 4),
            MessageEntity(MessageEntity.ITALIC, 12, 4),
        ]
        message = await bot.send_photo(
            chat_id, photo_file, caption=test_string, caption_entities=entities
        )

        assert message.caption == test_string
        assert message.caption_entities == tuple(entities)

    @pytest.mark.parametrize("default_bot", [{"parse_mode": "Markdown"}], indirect=True)
    async def test_send_photo_default_parse_mode_1(self, default_bot, chat_id, photo_file):
        test_string = "Italic Bold Code"
        test_markdown_string = "_Italic_ *Bold* `Code`"

        message = await default_bot.send_photo(chat_id, photo_file, caption=test_markdown_string)
        assert message.caption_markdown == test_markdown_string
        assert message.caption == test_string

    @pytest.mark.parametrize("default_bot", [{"parse_mode": "Markdown"}], indirect=True)
    async def test_send_photo_default_parse_mode_2(self, default_bot, chat_id, photo_file):
        test_markdown_string = "_Italic_ *Bold* `Code`"

        message = await default_bot.send_photo(
            chat_id, photo_file, caption=test_markdown_string, parse_mode=None
        )
        assert message.caption == test_markdown_string
        assert message.caption_markdown == escape_markdown(test_markdown_string)

    @pytest.mark.parametrize("default_bot", [{"parse_mode": "Markdown"}], indirect=True)
    async def test_send_photo_default_parse_mode_3(self, default_bot, chat_id, photo_file):
        test_markdown_string = "_Italic_ *Bold* `Code`"

        message = await default_bot.send_photo(
            chat_id, photo_file, caption=test_markdown_string, parse_mode="HTML"
        )
        assert message.caption == test_markdown_string
        assert message.caption_markdown == escape_markdown(test_markdown_string)

    @pytest.mark.parametrize("default_bot", [{"protect_content": True}], indirect=True)
    async def test_send_photo_default_protect_content(self, chat_id, default_bot, photo):
        tasks = asyncio.gather(
            default_bot.send_photo(chat_id, photo),
            default_bot.send_photo(chat_id, photo, protect_content=False),
        )
        protected, unprotected = await tasks
        assert protected.has_protected_content
        assert not unprotected.has_protected_content

    @pytest.mark.parametrize(
        ("default_bot", "custom"),
        [
            ({"allow_sending_without_reply": True}, None),
            ({"allow_sending_without_reply": False}, None),
            ({"allow_sending_without_reply": False}, True),
        ],
        indirect=["default_bot"],
    )
    async def test_send_photo_default_allow_sending_without_reply(
        self, default_bot, chat_id, photo_file, custom
    ):
        reply_to_message = await default_bot.send_message(chat_id, "test")
        await reply_to_message.delete()
        if custom is not None:
            message = await default_bot.send_photo(
                chat_id,
                photo_file,
                allow_sending_without_reply=custom,
                reply_to_message_id=reply_to_message.message_id,
            )
            assert message.reply_to_message is None
        elif default_bot.defaults.allow_sending_without_reply:
            message = await default_bot.send_photo(
                chat_id, photo_file, reply_to_message_id=reply_to_message.message_id
            )
            assert message.reply_to_message is None
        else:
            with pytest.raises(BadRequest, match="Message to be replied not found"):
                await default_bot.send_photo(
                    chat_id, photo_file, reply_to_message_id=reply_to_message.message_id
                )

    async def test_get_and_download(self, bot, photo, tmp_file):
        new_file = await bot.getFile(photo.file_id)

        assert new_file.file_size == photo.file_size
        assert new_file.file_unique_id == photo.file_unique_id
        assert new_file.file_path.startswith("https://") is True

        await new_file.download_to_drive(tmp_file)

        assert tmp_file.is_file()

    async def test_send_url_jpg_file(self, bot, chat_id):
        message = await bot.send_photo(chat_id, photo=self.photo_file_url)

        assert isinstance(message.photo[-2], PhotoSize)
        assert isinstance(message.photo[-2].file_id, str)
        assert isinstance(message.photo[-2].file_unique_id, str)
        assert message.photo[-2].file_id
        assert message.photo[-2].file_unique_id

        assert isinstance(message.photo[-1], PhotoSize)
        assert isinstance(message.photo[-1].file_id, str)
        assert isinstance(message.photo[-1].file_unique_id, str)
        assert message.photo[-1].file_id
        assert message.photo[-1].file_unique_id

    async def test_send_url_png_file(self, bot, chat_id):
        message = await bot.send_photo(
            photo="http://dummyimage.com/600x400/000/fff.png&text=telegram", chat_id=chat_id
        )

        photo = message.photo[-1]

        assert isinstance(photo, PhotoSize)
        assert isinstance(photo.file_id, str)
        assert isinstance(photo.file_unique_id, str)
        assert photo.file_id
        assert photo.file_unique_id

    async def test_send_file_unicode_filename(self, bot, chat_id):
        """
        Regression test for https://github.com/python-telegram-bot/python-telegram-bot/issues/1202
        """
        with data_file("测试.png").open("rb") as f:
            message = await bot.send_photo(photo=f, chat_id=chat_id)

        photo = message.photo[-1]

        assert isinstance(photo, PhotoSize)
        assert isinstance(photo.file_id, str)
        assert isinstance(photo.file_unique_id, str)
        assert photo.file_id
        assert photo.file_unique_id

    async def test_send_bytesio_jpg_file(self, bot, chat_id):
        filepath = data_file("telegram_no_standard_header.jpg")

        # raw image bytes
        raw_bytes = BytesIO(filepath.read_bytes())
        input_file = InputFile(raw_bytes)
        assert input_file.mimetype == "application/octet-stream"

        # raw image bytes with name info
        raw_bytes = BytesIO(filepath.read_bytes())
        raw_bytes.name = str(filepath)
        input_file = InputFile(raw_bytes)
        assert input_file.mimetype == "image/jpeg"

        # send raw photo
        raw_bytes = BytesIO(filepath.read_bytes())
        message = await bot.send_photo(chat_id, photo=raw_bytes)
        photo = message.photo[-1]
        assert isinstance(photo.file_id, str)
        assert isinstance(photo.file_unique_id, str)
        assert photo.file_id
        assert photo.file_unique_id
        assert isinstance(photo, PhotoSize)
        assert photo.width == 1280
        assert photo.height == 720
        assert photo.file_size == 33372

    async def test_resend(self, bot, chat_id, photo):
        message = await bot.send_photo(chat_id=chat_id, photo=photo.file_id)

        assert isinstance(message.photo[-2], PhotoSize)
        assert isinstance(message.photo[-2].file_id, str)
        assert isinstance(message.photo[-2].file_unique_id, str)
        assert message.photo[-2].file_id
        assert message.photo[-2].file_unique_id

        assert isinstance(message.photo[-1], PhotoSize)
        assert isinstance(message.photo[-1].file_id, str)
        assert isinstance(message.photo[-1].file_unique_id, str)
        assert message.photo[-1].file_id
        assert message.photo[-1].file_unique_id

    async def test_error_send_empty_file(self, bot, chat_id):
        with Path(os.devnull).open("rb") as file, pytest.raises(TelegramError):
            await bot.send_photo(chat_id=chat_id, photo=file)

    async def test_error_send_empty_file_id(self, bot, chat_id):
        with pytest.raises(TelegramError):
            await bot.send_photo(chat_id=chat_id, photo="")



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import (
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    InlineQueryResultPhoto,
    InlineQueryResultVoice,
    InputTextMessageContent,
    MessageEntity,
)
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def inline_query_result_photo():
    return InlineQueryResultPhoto(
        InlineQueryResultPhotoTestBase.id_,
        InlineQueryResultPhotoTestBase.photo_url,
        InlineQueryResultPhotoTestBase.thumbnail_url,
        photo_width=InlineQueryResultPhotoTestBase.photo_width,
        photo_height=InlineQueryResultPhotoTestBase.photo_height,
        title=InlineQueryResultPhotoTestBase.title,
        description=InlineQueryResultPhotoTestBase.description,
        caption=InlineQueryResultPhotoTestBase.caption,
        parse_mode=InlineQueryResultPhotoTestBase.parse_mode,
        caption_entities=InlineQueryResultPhotoTestBase.caption_entities,
        input_message_content=InlineQueryResultPhotoTestBase.input_message_content,
        reply_markup=InlineQueryResultPhotoTestBase.reply_markup,
        show_caption_above_media=InlineQueryResultPhotoTestBase.show_caption_above_media,
    )


class InlineQueryResultPhotoTestBase:
    id_ = "id"
    type_ = "photo"
    photo_url = "photo url"
    photo_width = 10
    photo_height = 15
    thumbnail_url = "thumb url"
    title = "title"
    description = "description"
    caption = "caption"
    parse_mode = "HTML"
    caption_entities = [MessageEntity(MessageEntity.ITALIC, 0, 7)]

    input_message_content = InputTextMessageContent("input_message_content")
    reply_markup = InlineKeyboardMarkup([[InlineKeyboardButton("reply_markup")]])
    show_caption_above_media = True


class TestInlineQueryResultPhotoWithoutRequest(InlineQueryResultPhotoTestBase):
    def test_slot_behaviour(self, inline_query_result_photo):
        inst = inline_query_result_photo
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, inline_query_result_photo):
        assert inline_query_result_photo.type == self.type_
        assert inline_query_result_photo.id == self.id_
        assert inline_query_result_photo.photo_url == self.photo_url
        assert inline_query_result_photo.photo_width == self.photo_width
        assert inline_query_result_photo.photo_height == self.photo_height
        assert inline_query_result_photo.thumbnail_url == self.thumbnail_url
        assert inline_query_result_photo.title == self.title
        assert inline_query_result_photo.description == self.description
        assert inline_query_result_photo.caption == self.caption
        assert inline_query_result_photo.parse_mode == self.parse_mode
        assert inline_query_result_photo.caption_entities == tuple(self.caption_entities)
        assert (
            inline_query_result_photo.input_message_content.to_dict()
            == self.input_message_content.to_dict()
        )
        assert inline_query_result_photo.reply_markup.to_dict() == self.reply_markup.to_dict()
        assert inline_query_result_photo.show_caption_above_media == self.show_caption_above_media

    def test_caption_entities_always_tuple(self):
        result = InlineQueryResultPhoto(self.id_, self.photo_url, self.thumbnail_url)
        assert result.caption_entities == ()

    def test_to_dict(self, inline_query_result_photo):
        inline_query_result_photo_dict = inline_query_result_photo.to_dict()

        assert isinstance(inline_query_result_photo_dict, dict)
        assert inline_query_result_photo_dict["type"] == inline_query_result_photo.type
        assert inline_query_result_photo_dict["id"] == inline_query_result_photo.id
        assert inline_query_result_photo_dict["photo_url"] == inline_query_result_photo.photo_url
        assert (
            inline_query_result_photo_dict["photo_width"] == inline_query_result_photo.photo_width
        )
        assert (
            inline_query_result_photo_dict["photo_height"]
            == inline_query_result_photo.photo_height
        )
        assert (
            inline_query_result_photo_dict["thumbnail_url"]
            == inline_query_result_photo.thumbnail_url
        )
        assert inline_query_result_photo_dict["title"] == inline_query_result_photo.title
        assert (
            inline_query_result_photo_dict["description"] == inline_query_result_photo.description
        )
        assert inline_query_result_photo_dict["caption"] == inline_query_result_photo.caption
        assert inline_query_result_photo_dict["parse_mode"] == inline_query_result_photo.parse_mode
        assert inline_query_result_photo_dict["caption_entities"] == [
            ce.to_dict() for ce in inline_query_result_photo.caption_entities
        ]
        assert (
            inline_query_result_photo_dict["input_message_content"]
            == inline_query_result_photo.input_message_content.to_dict()
        )
        assert (
            inline_query_result_photo_dict["reply_markup"]
            == inline_query_result_photo.reply_markup.to_dict()
        )
        assert (
            inline_query_result_photo_dict["show_caption_above_media"]
            == inline_query_result_photo.show_caption_above_media
        )

    def test_equality(self):
        a = InlineQueryResultPhoto(self.id_, self.photo_url, self.thumbnail_url)
        b = InlineQueryResultPhoto(self.id_, self.photo_url, self.thumbnail_url)
        c = InlineQueryResultPhoto(self.id_, "", self.thumbnail_url)
        d = InlineQueryResultPhoto("", self.photo_url, self.thumbnail_url)
        e = InlineQueryResultVoice(self.id_, "", "")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import PhotoSize, VideoQuality
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def video_quality_message_id():
    # A video message with exactly one available video quality:
    # https://t.me/pythontelegrambottests/375821
    # See discussion: https://t.me/pythontelegrambotdev/1782
    return 375821


@pytest.fixture(scope="module")
async def video_quality_list(bot, chat_id, channel_id, video_quality_message_id):
    return (
        await bot.forward_message(chat_id, channel_id, video_quality_message_id, read_timeout=50)
    ).video.qualities


@pytest.fixture(scope="module")
def video_quality(video_quality_list):
    return video_quality_list[-1]


class VideoQualityTestBase:
    # These values are tied to the forwarded video and must remain unchanged
    width = 464
    height = 848
    codec = "h264"
    file_size = 405540


class TestVideoQualityWithoutRequest(VideoQualityTestBase):
    def test_qualities_available(self, video_quality_list):
        assert isinstance(video_quality_list, tuple)
        # Subsequent tests relie on the forwarded video
        # having exactly one video quality.
        assert len(video_quality_list) == 1

    def test_slot_behaviour(self, video_quality):
        for attr in video_quality.__slots__:
            assert getattr(video_quality, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(video_quality)) == len(set(mro_slots(video_quality))), (
            "duplicate slot"
        )

    def test_creation(self, video_quality):
        assert isinstance(video_quality, VideoQuality)
        assert isinstance(video_quality.file_id, str)
        assert isinstance(video_quality.file_unique_id, str)
        assert video_quality.file_id
        assert video_quality.file_unique_id

    def test_expected_values(self, video_quality):
        assert video_quality.width == self.width
        assert video_quality.height == self.height
        assert video_quality.codec == self.codec
        assert video_quality.file_size == self.file_size

    def test_de_json(self, offline_bot, video_quality):
        json_dict = {
            "file_id": video_quality.file_id,
            "file_unique_id": video_quality.file_unique_id,
            "width": self.width,
            "height": self.height,
            "codec": self.codec,
            "file_size": self.file_size,
        }
        json_videoquality = VideoQuality.de_json(json_dict, offline_bot)
        assert json_videoquality.api_kwargs == {}

        assert json_videoquality.file_id == video_quality.file_id
        assert json_videoquality.file_unique_id == video_quality.file_unique_id
        assert json_videoquality.width == self.width
        assert json_videoquality.height == self.height
        assert json_videoquality.codec == self.codec
        assert json_videoquality.file_size == self.file_size

    def test_to_dict(self, video_quality):
        videoquality_dict = video_quality.to_dict()

        assert isinstance(videoquality_dict, dict)
        assert videoquality_dict["file_id"] == video_quality.file_id
        assert videoquality_dict["file_unique_id"] == video_quality.file_unique_id
        assert videoquality_dict["width"] == video_quality.width
        assert videoquality_dict["height"] == video_quality.height
        assert videoquality_dict["codec"] == video_quality.codec
        assert videoquality_dict["file_size"] == video_quality.file_size

    def test_equality(self, video_quality):
        a = VideoQuality(
            video_quality.file_id,
            video_quality.file_unique_id,
            self.width,
            self.height,
            self.codec,
        )
        b = VideoQuality("", video_quality.file_unique_id, self.width, self.height, self.codec)
        c = VideoQuality(video_quality.file_id, video_quality.file_unique_id, 0, 0, self.codec)
        d = VideoQuality("", "", self.width, self.height, self.codec)
        e = PhotoSize(
            video_quality.file_id,
            video_quality.file_unique_id,
            self.width,
            self.height,
        )

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)


class TestVideoWithRequest(VideoQualityTestBase):
    async def test_get_and_download(self, bot, video_quality, tmp_file):
        new_file = await bot.get_file(video_quality.file_id)

        assert new_file.file_size == video_quality.file_size
        assert new_file.file_unique_id == video_quality.file_unique_id
        assert new_file.file_path.startswith("https://")

        await new_file.download_to_drive(tmp_file)

        assert tmp_file.is_file()

    async def test_resend(self, bot, chat_id, video_quality):
        message = await bot.send_video(chat_id, video_quality.file_id)

        assert message.video.file_id == video_quality.file_id
        assert message.video.file_unique_id == video_quality.file_unique_id



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""Base class for Telegram Objects."""

import contextlib
import datetime as dtm
import inspect
import json
from collections.abc import Iterator, Mapping, Sized
from contextlib import contextmanager
from copy import deepcopy
from itertools import chain
from types import MappingProxyType
from typing import TYPE_CHECKING, Any, ClassVar, TypeVar, cast

from telegram._utils.datetime import to_timestamp
from telegram._utils.defaultvalue import DefaultValue
from telegram._utils.types import JSONDict
from telegram._utils.warnings import warn

if TYPE_CHECKING:
    from telegram import Bot

Tele_co = TypeVar("Tele_co", bound="TelegramObject", covariant=True)
Tele = TypeVar("Tele", bound="TelegramObject")


class TelegramObject:
    """Base class for most Telegram objects.

    Objects of this type are subscriptable with strings. See :meth:`__getitem__` for more details.
    The :mod:`pickle` and :func:`~copy.deepcopy` behavior of objects of this type are defined by
    :meth:`__getstate__`, :meth:`__setstate__` and :meth:`__deepcopy__`.

    Tip:
        Objects of this type can be serialized via Python's :mod:`pickle` module and pickled
        objects from one version of PTB are usually loadable in future versions. However, we can
        not guarantee that this compatibility will always be provided. At least a manual one-time
        conversion of the data may be needed on major updates of the library.

    .. versionchanged:: 20.0

        * Removed argument and attribute ``bot`` for several subclasses. Use
          :meth:`set_bot` and :meth:`get_bot` instead.
        * Removed the possibility to pass arbitrary keyword arguments for several subclasses.
        * String representations objects of this type was overhauled. See :meth:`__repr__` for
          details. As this class doesn't implement :meth:`object.__str__`, the default
          implementation will be used, which is equivalent to :meth:`__repr__`.
        * Objects of this class (or subclasses) are now immutable. This means that you can't set
          or delete attributes anymore. Moreover, attributes that were formerly of type
          :obj:`list` are now of type :obj:`tuple`.

    Arguments:
        api_kwargs (dict[:obj:`str`, any], optional): |toapikwargsarg|

            .. versionadded:: 20.0

    Attributes:
        api_kwargs (:obj:`types.MappingProxyType` [:obj:`str`, any]): |toapikwargsattr|

            .. versionadded:: 20.0

    """

    __slots__ = ("_bot", "_frozen", "_id_attrs", "api_kwargs")

    # Used to cache the names of the parameters of the __init__ method of the class
    # Must be a private attribute to avoid name clashes between subclasses
    __INIT_PARAMS: ClassVar[set[str]] = set()
    # Used to check if __INIT_PARAMS has been set for the current class. Unfortunately, we can't
    # just check if `__INIT_PARAMS is None`, since subclasses use the parent class' __INIT_PARAMS
    # unless it's overridden
    __INIT_PARAMS_CHECK: type["TelegramObject"] | None = None

    def __init__(self, *, api_kwargs: JSONDict | None = None) -> None:
        # Setting _frozen to `False` here means that classes without arguments still need to
        # implement __init__. However, with `True` would mean increased usage of
        # `with self._unfrozen()` in the `__init__` of subclasses and we have fewer empty
        # classes than classes with arguments.
        self._frozen: bool = False
        self._id_attrs: tuple[object, ...] = ()
        self._bot: Bot | None = None
        # We don't do anything with api_kwargs here - see docstring of _apply_api_kwargs
        self.api_kwargs: Mapping[str, Any] = MappingProxyType(api_kwargs or {})

    def __eq__(self, other: object) -> bool:
        """Compares this object with :paramref:`other` in terms of equality.
        If this object and :paramref:`other` are `not` objects of the same class,
        this comparison will fall back to Python's default implementation of :meth:`object.__eq__`.
        Otherwise, both objects may be compared in terms of equality, if the corresponding
        subclass of :class:`TelegramObject` has defined a set of attributes to compare and
        the objects are considered to be equal, if all of these attributes are equal.
        If the subclass has not defined a set of attributes to compare, a warning will be issued.

        Tip:
            If instances of a class in the :mod:`telegram` module are comparable in terms of
            equality, the documentation of the class will state the attributes that will be used
            for this comparison.

        Args:
            other (:obj:`object`): The object to compare with.

        Returns:
            :obj:`bool`

        """
        if isinstance(other, self.__class__):
            if not self._id_attrs:
                warn(
                    f"Objects of type {self.__class__.__name__} can not be meaningfully tested for"
                    " equivalence.",
                    stacklevel=2,
                )
            if not other._id_attrs:
                warn(
                    f"Objects of type {other.__class__.__name__} can not be meaningfully tested"
                    " for equivalence.",
                    stacklevel=2,
                )
            return self._id_attrs == other._id_attrs
        return super().__eq__(other)

    def __hash__(self) -> int:
        """Builds a hash value for this object such that the hash of two objects is equal if and
        only if the objects are equal in terms of :meth:`__eq__`.

        Returns:
            :obj:`int`
        """
        if self._id_attrs:
            return hash((self.__class__, self._id_attrs))
        return super().__hash__()

    def __setattr__(self, key: str, value: object) -> None:
        """Overrides :meth:`object.__setattr__` to prevent the overriding of attributes.

        Raises:
            :exc:`AttributeError`
        """
        # protected attributes can always be set for convenient internal use
        if key[0] == "_" or not getattr(self, "_frozen", True):
            super().__setattr__(key, value)
            return

        raise AttributeError(
            f"Attribute `{key}` of class `{self.__class__.__name__}` can't be set!"
        )

    def __delattr__(self, key: str) -> None:
        """Overrides :meth:`object.__delattr__` to prevent the deletion of attributes.

        Raises:
            :exc:`AttributeError`
        """
        # protected attributes can always be set for convenient internal use
        if key[0] == "_" or not getattr(self, "_frozen", True):
            super().__delattr__(key)
            return

        raise AttributeError(
            f"Attribute `{key}` of class `{self.__class__.__name__}` can't be deleted!"
        )

    def __repr__(self) -> str:
        """Gives a string representation of this object in the form
        ``ClassName(attr_1=value_1, attr_2=value_2, ...)``, where attributes are omitted if they
        have the value :obj:`None` or are empty instances of :class:`collections.abc.Sized` (e.g.
        :class:`list`, :class:`dict`, :class:`set`, :class:`str`, etc.).

        As this class doesn't implement :meth:`object.__str__`, the default implementation
        will be used, which is equivalent to :meth:`__repr__`.

        Returns:
            :obj:`str`
        """
        # * `__repr__` goal is to be unambiguous
        # * `__str__` goal is to be readable
        # * `str()` calls `__repr__`, if `__str__` is not defined
        # In our case "unambiguous" and "readable" largely coincide, so we can use the same logic.
        as_dict = self._get_attrs(recursive=False, include_private=False)

        if not self.api_kwargs:
            # Drop api_kwargs from the representation, if empty
            as_dict.pop("api_kwargs", None)
        else:
            # Otherwise, we want to skip the "mappingproxy" part of the repr
            as_dict["api_kwargs"] = dict(self.api_kwargs)

        contents = ", ".join(
            f"{k}={as_dict[k]!r}"
            for k in sorted(as_dict.keys())
            if (
                as_dict[k] is not None
                and not (
                    isinstance(as_dict[k], Sized) and len(as_dict[k]) == 0  # type: ignore[arg-type]
                )
            )
        )
        return f"{self.__class__.__name__}({contents})"

    def __getitem__(self, item: str) -> object:
        """
        Objects of this type are subscriptable with strings, where
        ``telegram_object["attribute_name"]`` is equivalent to ``telegram_object.attribute_name``.

        Tip:
            This is useful for dynamic attribute lookup, i.e. ``telegram_object[arg]`` where the
            value of ``arg`` is determined at runtime.
            In all other cases, it's recommended to use the dot notation instead, i.e.
            ``telegram_object.attribute_name``.

        .. versionchanged:: 20.0

            ``telegram_object['from']`` will look up the key ``from_user``. This is to account for
            special cases like :attr:`Message.from_user` that deviate from the official Bot API.

        Args:
            item (:obj:`str`): The name of the attribute to look up.

        Returns:
            :obj:`object`

        Raises:
            :exc:`KeyError`: If the object does not have an attribute with the appropriate name.
        """
        if item == "from":
            item = "from_user"
        try:
            return getattr(self, item)
        except AttributeError as exc:
            raise KeyError(
                f"Objects of type {self.__class__.__name__} don't have an attribute called "
                f"`{item}`."
            ) from exc

    def __getstate__(self) -> dict[str, str | object]:
        """
        Overrides :meth:`object.__getstate__` to customize the pickling process of objects of this
        type.
        The returned state does `not` contain the :class:`telegram.Bot` instance set with
        :meth:`set_bot` (if any), as it can't be pickled.

        Returns:
            state (dict[:obj:`str`, :obj:`object`]): The state of the object.
        """
        out = self._get_attrs(
            include_private=True, recursive=False, remove_bot=True, convert_default_value=False
        )
        # MappingProxyType is not pickable, so we convert it to a dict and revert in
        # __setstate__
        out["api_kwargs"] = dict(self.api_kwargs)
        return out

    def __setstate__(self, state: dict[str, object]) -> None:
        """
        Overrides :meth:`object.__setstate__` to customize the unpickling process of objects of
        this type. Modifies the object in-place.

        If any data was stored in the :attr:`api_kwargs` of the pickled object, this method checks
        if the class now has dedicated attributes for those keys and moves the values from
        :attr:`api_kwargs` to the dedicated attributes.
        This can happen, if serialized data is loaded with a new version of this library, where
        the new version was updated to account for updates of the Telegram Bot API.

        If on the contrary an attribute was removed from the class, the value is not discarded but
        made available via :attr:`api_kwargs`.

        Args:
            state (:obj:`dict`): The data to set as attributes of this object.
        """
        self._unfreeze()

        # Make sure that we have a `_bot` attribute. This is necessary, since __getstate__ omits
        # this as Bots are not pickable.
        self._bot = None

        # get api_kwargs first because we may need to add entries to it (see try-except below)
        api_kwargs = cast("dict[str, object]", state.pop("api_kwargs", {}))
        # get _frozen before the loop to avoid setting it to True in the loop
        frozen = state.pop("_frozen", False)

        for key, val in state.items():
            try:
                setattr(self, key, val)
            except AttributeError:
                # So an attribute was deprecated and removed from the class. Let's handle this:
                # 1) Is the attribute now a property with no setter? Let's check that:
                if isinstance(getattr(self.__class__, key, None), property):
                    # It is, so let's try to set the "private attribute" instead
                    try:
                        setattr(self, f"_{key}", val)
                    # If this fails as well, guess we've completely removed it. Let's add it to
                    # api_kwargs as fallback
                    except AttributeError:
                        api_kwargs[key] = val

                # 2) The attribute is a private attribute, i.e. it went through case 1) in the past
                elif key.startswith("_"):
                    continue  # skip adding this to api_kwargs, the attribute is lost forever.
                api_kwargs[key] = val  # add it to api_kwargs as fallback

        # For api_kwargs we first apply any kwargs that are already attributes of the object
        # and then set the rest as MappingProxyType attribute. Converting to MappingProxyType
        # is necessary, since __getstate__ converts it to a dict as MPT is not pickable.
        self._apply_api_kwargs(api_kwargs)
        self.api_kwargs = MappingProxyType(api_kwargs)

        # Apply freezing if necessary
        # we .get(…) the setting for backwards compatibility with objects that were pickled
        # before the freeze feature was introduced
        if frozen:
            self._freeze()

    def __deepcopy__(self: Tele_co, memodict: dict[int, object]) -> Tele_co:
        """
        Customizes how :func:`copy.deepcopy` processes objects of this type.
        The only difference to the default implementation is that the :class:`telegram.Bot`
        instance set via :meth:`set_bot` (if any) is not copied, but shared between the original
        and the copy, i.e.::

            assert telegram_object.get_bot() is copy.deepcopy(telegram_object).get_bot()

        Args:
            memodict (:obj:`dict`): A dictionary that maps objects to their copies.

        Returns:
            :class:`telegram.TelegramObject`: The copied object.
        """
        bot = self._bot  # Save bot so we can set it after copying
        self.set_bot(None)  # set to None so it is not deepcopied
        cls = self.__class__
        result = cls.__new__(cls)  # create a new instance
        memodict[id(self)] = result  # save the id of the object in the dict

        result._frozen = False  # unfreeze the new object for setting the attributes

        # now we set the attributes in the deepcopied object
        for k in self._get_attrs_names(include_private=True):
            if k == "_frozen":
                # Setting the frozen status to True would prevent the attributes from being set
                continue
            if k == "api_kwargs":
                # Need to copy api_kwargs manually, since it's a MappingProxyType is not
                # pickable and deepcopy uses the pickle interface
                setattr(result, k, MappingProxyType(deepcopy(dict(self.api_kwargs), memodict)))
                continue

            try:
                setattr(result, k, deepcopy(getattr(self, k), memodict))
            except AttributeError:
                # Skip missing attributes. This can happen if the object was loaded from a pickle
                # file that was created with an older version of the library, where the class
                # did not have the attribute yet.
                continue

        # Apply freezing if necessary
        if self._frozen:
            result._freeze()

        result.set_bot(bot)  # Assign the bots back
        self.set_bot(bot)
        return result

    @staticmethod
    def _parse_data(data: JSONDict) -> JSONDict:
        """Should be called by subclasses that override de_json to ensure that the input
        is not altered. Whoever calls de_json might still want to use the original input
        for something else.
        """
        return data.copy()

    @classmethod
    def _de_json(
        cls: type[Tele_co],
        data: JSONDict,
        bot: "Bot | None",
        api_kwargs: JSONDict | None = None,
    ) -> Tele_co:
        # try-except is significantly faster in case we already have a correct argument set
        try:
            obj = cls(**data, api_kwargs=api_kwargs)
        except TypeError as exc:
            if "__init__() got an unexpected keyword argument" not in str(exc):
                raise

            if cls.__INIT_PARAMS_CHECK is not cls:
                signature = inspect.signature(cls)
                cls.__INIT_PARAMS = set(signature.parameters.keys())
                cls.__INIT_PARAMS_CHECK = cls

            api_kwargs = api_kwargs or {}
            existing_kwargs: JSONDict = {}
            for key, value in data.items():
                (existing_kwargs if key in cls.__INIT_PARAMS else api_kwargs)[key] = value

            obj = cls(api_kwargs=api_kwargs, **existing_kwargs)

        obj.set_bot(bot=bot)
        return obj

    @classmethod
    def de_json(cls: type[Tele_co], data: JSONDict, bot: "Bot | None" = None) -> Tele_co:
        """Converts JSON data to a Telegram object.

        Args:
            data (dict[:obj:`str`, ...]): The JSON data.
            bot (:class:`telegram.Bot`, optional): The bot associated with this object. Defaults to
                :obj:`None`, in which case shortcut methods will not be available.

                .. versionchanged:: 21.4
                   :paramref:`bot` is now optional and defaults to :obj:`None`

        Returns:
            The Telegram object.

        """
        return cls._de_json(data=data, bot=bot)

    @classmethod
    def de_list(
        cls: type[Tele_co], data: list[JSONDict], bot: "Bot | None" = None
    ) -> tuple[Tele_co, ...]:
        """Converts a list of JSON objects to a tuple of Telegram objects.

        .. versionchanged:: 20.0

           * Returns a tuple instead of a list.
           * Filters out any :obj:`None` values.

        Args:
            data (list[dict[:obj:`str`, ...]]): The JSON data.
            bot (:class:`telegram.Bot`, optional): The bot associated with these object. Defaults
                to :obj:`None`, in which case shortcut methods will not be available.

                .. versionchanged:: 21.4
                   :paramref:`bot` is now optional and defaults to :obj:`None`

        Returns:
            A tuple of Telegram objects.

        """
        return tuple(cls.de_json(d, bot) for d in data)

    @contextmanager
    def _unfrozen(self: Tele) -> Iterator[Tele]:
        """Context manager to temporarily unfreeze the object. For internal use only.

        Note:
            with to._unfrozen() as other_to:
                assert to is other_to
        """
        self._unfreeze()
        yield self
        self._freeze()

    def _freeze(self) -> None:
        self._frozen = True

    def _unfreeze(self) -> None:
        self._frozen = False

    def _apply_api_kwargs(self, api_kwargs: JSONDict) -> None:
        """Loops through the api kwargs and for every key that exists as attribute of the
        object (and is None), it moves the value from `api_kwargs` to the attribute.
        *Edits `api_kwargs` in place!*

        This method is currently only called in the unpickling process, i.e. not on "normal" init.
        This is because
        * automating this is tricky to get right: It should be called at the *end* of the __init__,
          preferably only once at the end of the __init__ of the last child class. This could be
          done via __init_subclass__, but it's hard to not destroy the signature of __init__ in the
          process.
        * calling it manually in every __init__ is tedious
        * There probably is no use case for it anyway. If you manually initialize a TO subclass,
          then you can pass everything as proper argument.
        """
        # we convert to list to ensure that the list doesn't change length while we loop
        for key in list(api_kwargs.keys()):
            # property attributes are not settable, so we need to set the private attribute
            if isinstance(getattr(self.__class__, key, None), property):
                # if setattr fails, we'll just leave the value in api_kwargs:
                with contextlib.suppress(AttributeError):
                    setattr(self, f"_{key}", api_kwargs.pop(key))
            elif getattr(self, key, True) is None:
                setattr(self, key, api_kwargs.pop(key))

    def _is_deprecated_attr(self, attr: str) -> bool:
        """Checks whether `attr` is in the list of deprecated time period attributes."""
        return (
            class_name := self.__class__.__name__
        ) in _TIME_PERIOD_DEPRECATIONS and attr in _TIME_PERIOD_DEPRECATIONS[class_name]

    def _get_attrs_names(self, include_private: bool) -> Iterator[str]:
        """
        Returns the names of the attributes of this object. This is used to determine which
        attributes should be serialized when pickling the object.

        Args:
            include_private (:obj:`bool`): Whether to include private attributes.

        Returns:
            Iterator[:obj:`str`]: An iterator over the names of the attributes of this object.
        """
        # We want to get all attributes for the class, using self.__slots__ only includes the
        # attributes used by that class itself, and not its superclass(es). Hence, we get its MRO
        # and then get their attributes. The `[:-1]` slice excludes the `object` class
        all_slots = (s for c in self.__class__.__mro__[:-1] for s in c.__slots__)  # type: ignore
        # chain the class's slots with the user defined subclass __dict__ (class has no slots)
        all_attrs = (
            chain(all_slots, self.__dict__.keys()) if hasattr(self, "__dict__") else all_slots
        )

        if include_private:
            return all_attrs
        return (
            attr
            for attr in all_attrs
            # Include deprecated private attributes, which are exposed via properties
            if not attr.startswith("_") or self._is_deprecated_attr(attr)
        )

    def _get_attrs(
        self,
        include_private: bool = False,
        recursive: bool = False,
        remove_bot: bool = False,
        convert_default_value: bool = True,
    ) -> dict[str, str | object]:
        """This method is used for obtaining the attributes of the object.

        Args:
            include_private (:obj:`bool`): Whether the result should include private variables.
            recursive (:obj:`bool`): If :obj:`True`, will convert any ``TelegramObjects`` (if
                found) in the attributes to a dictionary. Else, preserves it as an object itself.
            remove_bot (:obj:`bool`): Whether the bot should be included in the result.
            convert_default_value (:obj:`bool`): Whether :class:`telegram.DefaultValue` should be
                converted to its true value. This is necessary when converting to a dictionary for
                end users since DefaultValue is used in some classes that work with
                `tg.ext.defaults` (like `LinkPreviewOptions`)

        Returns:
            :obj:`dict`: A dict where the keys are attribute names and values are their values.
        """
        data = {}

        for key in self._get_attrs_names(include_private=include_private):
            value = (
                DefaultValue.get_value(getattr(self, key, None))
                if convert_default_value
                else getattr(self, key, None)
            )

            if value is not None:
                if recursive and hasattr(value, "to_dict"):
                    data[key] = value.to_dict(recursive=True)
                else:
                    data[key] = value
            elif not recursive:
                data[key] = value

        if recursive and data.get("from_user"):
            data["from"] = data.pop("from_user", None)
        if remove_bot:
            data.pop("_bot", None)
        return data

    def to_json(self) -> str:
        """Gives a JSON representation of object.

        .. versionchanged:: 20.0
            Now includes all entries of :attr:`api_kwargs`.

        Returns:
            :obj:`str`
        """
        return json.dumps(self.to_dict())

    def to_dict(self, recursive: bool = True) -> JSONDict:
        """Gives representation of object as :obj:`dict`.

        .. versionchanged:: 20.0

            * Now includes all entries of :attr:`api_kwargs`.
            * Attributes whose values are empty sequences are no longer included.

        Args:
            recursive (:obj:`bool`, optional): If :obj:`True`, will convert any TelegramObjects
                (if found) in the attributes to a dictionary. Else, preserves it as an object
                itself. Defaults to :obj:`True`.

                .. versionadded:: 20.0

        Returns:
            :obj:`dict`
        """
        out = self._get_attrs(recursive=recursive)

        # Now we should convert TGObjects to dicts inside objects such as sequences, and convert
        # datetimes to timestamps. This mostly eliminates the need for subclasses to override
        # `to_dict`
        pop_keys: set[str] = set()
        timedelta_dict: dict = {}
        for key, value in out.items():
            if isinstance(value, tuple | list):
                if not value:
                    # not popping directly to avoid changing the dict size during iteration
                    pop_keys.add(key)
                    continue

                val = []  # empty list to append our converted values to
                for item in value:
                    if hasattr(item, "to_dict"):
                        val.append(item.to_dict(recursive=recursive))
                    # This branch is useful for e.g. tuple[tuple[PhotoSize|KeyboardButton]]
                    elif isinstance(item, tuple | list):
                        val.append(
                            [
                                i.to_dict(recursive=recursive) if hasattr(i, "to_dict") else i
                                for i in item
                            ]
                        )
                    else:  # if it's not a TGObject, just append it. E.g. [TGObject, 2]
                        val.append(item)
                out[key] = val

            elif isinstance(value, dtm.datetime):
                out[key] = to_timestamp(value)
            elif isinstance(value, dtm.timedelta):
                # Converting to int here is neccassry in some cases where Bot API returns
                # 'BadRquest' when expecting integers (e.g. InputMediaVideo.duration).
                # Other times, floats are accepted but the Bot API handles ints just as well
                # (e.g. InputStoryContentVideo.duration).
                # Not updating `out` directly to avoid changing the dict size during iteration
                timedelta_dict[key.removeprefix("_")] = (
                    int(seconds) if (seconds := value.total_seconds()).is_integer() else seconds
                )
                # This will sometimes add non-deprecated timedelta attributes to pop_keys.
                # We'll restore them shortly.
                pop_keys.add(key)

        for key in pop_keys:
            out.pop(key)

        # `out.update` must to be called *after* we pop deprecated time period attributes
        # this ensures that we restore attributes that were already using datetime.timdelta
        out.update(timedelta_dict)

        # Effectively "unpack" api_kwargs into `out`:
        out.update(out.pop("api_kwargs", {}))  # type: ignore[call-overload]
        return out

    def get_bot(self) -> "Bot":
        """Returns the :class:`telegram.Bot` instance associated with this object.

        .. seealso:: :meth:`set_bot`

        .. versionadded: 20.0

        Raises:
            RuntimeError: If no :class:`telegram.Bot` instance was set for this object.
        """
        if self._bot is None:
            raise RuntimeError(
                "This object has no bot associated with it. Shortcuts cannot be used."
            )
        return self._bot

    def set_bot(self, bot: "Bot | None") -> None:
        """Sets the :class:`telegram.Bot` instance associated with this object.

        .. seealso:: :meth:`get_bot`

        .. versionadded: 20.0

        Arguments:
            bot (:class:`telegram.Bot` | :obj:`None`): The bot instance.
        """
        self._bot = bot


# We use str keys to avoid importing which causes circular dependencies
_TIME_PERIOD_DEPRECATIONS: dict[str, tuple[str, ...]] = {
    "ChatFullInfo": ("_message_auto_delete_time", "_slow_mode_delay"),
    "Animation": ("_duration",),
    "Audio": ("_duration",),
    "Video": ("_duration", "_start_timestamp"),
    "VideoNote": ("_duration",),
    "Voice": ("_duration",),
    "PaidMediaPreview": ("_duration",),
    "VideoChatEnded": ("_duration",),
    "InputMediaVideo": ("_duration",),
    "InputMediaAnimation": ("_duration",),
    "InputMediaAudio": ("_duration",),
    "InputPaidMediaVideo": ("_duration",),
    "InlineQueryResultGif": ("_gif_duration",),
    "InlineQueryResultMpeg4Gif": ("_mpeg4_duration",),
    "InlineQueryResultVideo": ("_video_duration",),
    "InlineQueryResultAudio": ("_audio_duration",),
    "InlineQueryResultVoice": ("_voice_duration",),
    "InlineQueryResultLocation": ("_live_period",),
    "Poll": ("_open_period",),
    "Location": ("_live_period",),
    "MessageAutoDeleteTimerChanged": ("_message_auto_delete_time",),
    "ChatInviteLink": ("_subscription_period",),
    "InputLocationMessageContent": ("_live_period",),
}



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains objects related to chat backgrounds."""

from collections.abc import Sequence
from typing import TYPE_CHECKING, Final

from telegram import constants
from telegram._files.document import Document
from telegram._telegramobject import TelegramObject
from telegram._utils import enum
from telegram._utils.argumentparsing import de_json_optional, parse_sequence_arg
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class BackgroundFill(TelegramObject):
    """Base class for Telegram BackgroundFill Objects. It can be one of:

    * :class:`telegram.BackgroundFillSolid`
    * :class:`telegram.BackgroundFillGradient`
    * :class:`telegram.BackgroundFillFreeformGradient`

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`type` is equal.

    .. versionadded:: 21.2

    Args:
        type (:obj:`str`): Type of the background fill. Can be one of:
            :attr:`~telegram.BackgroundFill.SOLID`, :attr:`~telegram.BackgroundFill.GRADIENT`
            or :attr:`~telegram.BackgroundFill.FREEFORM_GRADIENT`.

    Attributes:
        type (:obj:`str`): Type of the background fill. Can be one of:
            :attr:`~telegram.BackgroundFill.SOLID`, :attr:`~telegram.BackgroundFill.GRADIENT`
            or :attr:`~telegram.BackgroundFill.FREEFORM_GRADIENT`.
    """

    __slots__ = ("type",)

    SOLID: Final[constants.BackgroundFillType] = constants.BackgroundFillType.SOLID
    """:const:`telegram.constants.BackgroundFillType.SOLID`"""
    GRADIENT: Final[constants.BackgroundFillType] = constants.BackgroundFillType.GRADIENT
    """:const:`telegram.constants.BackgroundFillType.GRADIENT`"""
    FREEFORM_GRADIENT: Final[constants.BackgroundFillType] = (
        constants.BackgroundFillType.FREEFORM_GRADIENT
    )
    """:const:`telegram.constants.BackgroundFillType.FREEFORM_GRADIENT`"""

    def __init__(
        self,
        type: str,  # pylint: disable=redefined-builtin
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required by all subclasses
        self.type: str = enum.get_member(constants.BackgroundFillType, type, type)

        self._id_attrs = (self.type,)
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "BackgroundFill":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        _class_mapping: dict[str, type[BackgroundFill]] = {
            cls.SOLID: BackgroundFillSolid,
            cls.GRADIENT: BackgroundFillGradient,
            cls.FREEFORM_GRADIENT: BackgroundFillFreeformGradient,
        }

        if cls is BackgroundFill and data.get("type") in _class_mapping:
            return _class_mapping[data.pop("type")].de_json(data=data, bot=bot)

        return super().de_json(data=data, bot=bot)


class BackgroundFillSolid(BackgroundFill):
    """
    The background is filled using the selected color.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`color` is equal.

    .. versionadded:: 21.2

    Args:
        color (:obj:`int`): The color of the background fill in the `RGB24` format.

    Attributes:
        type (:obj:`str`): Type of the background fill. Always
            :attr:`~telegram.BackgroundFill.SOLID`.
        color (:obj:`int`): The color of the background fill in the `RGB24` format.
    """

    __slots__ = ("color",)

    def __init__(
        self,
        color: int,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(type=self.SOLID, api_kwargs=api_kwargs)

        with self._unfrozen():
            self.color: int = color

            self._id_attrs = (self.color,)


class BackgroundFillGradient(BackgroundFill):
    """
    The background is a gradient fill.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`top_color`, :attr:`bottom_color`
    and :attr:`rotation_angle` are equal.

    .. versionadded:: 21.2

    Args:
        top_color (:obj:`int`): Top color of the gradient in the `RGB24` format.
        bottom_color (:obj:`int`): Bottom color of the gradient in the `RGB24` format.
        rotation_angle (:obj:`int`): Clockwise rotation angle of the background
            fill in degrees;
            0-:tg-const:`telegram.constants.BackgroundFillLimit.MAX_ROTATION_ANGLE`.


    Attributes:
        type (:obj:`str`): Type of the background fill. Always
            :attr:`~telegram.BackgroundFill.GRADIENT`.
        top_color (:obj:`int`): Top color of the gradient in the `RGB24` format.
        bottom_color (:obj:`int`): Bottom color of the gradient in the `RGB24` format.
        rotation_angle (:obj:`int`): Clockwise rotation angle of the background
            fill in degrees;
            0-:tg-const:`telegram.constants.BackgroundFillLimit.MAX_ROTATION_ANGLE`.
    """

    __slots__ = ("bottom_color", "rotation_angle", "top_color")

    def __init__(
        self,
        top_color: int,
        bottom_color: int,
        rotation_angle: int,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(type=self.GRADIENT, api_kwargs=api_kwargs)

        with self._unfrozen():
            self.top_color: int = top_color
            self.bottom_color: int = bottom_color
            self.rotation_angle: int = rotation_angle

            self._id_attrs = (self.top_color, self.bottom_color, self.rotation_angle)


class BackgroundFillFreeformGradient(BackgroundFill):
    """
    The background is a freeform gradient that rotates after every message in the chat.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`colors` is equal.

    .. versionadded:: 21.2

    Args:
        colors (Sequence[:obj:`int`]): A list of the 3 or 4 base colors that are used to
            generate the freeform gradient in the `RGB24` format

    Attributes:
        type (:obj:`str`): Type of the background fill. Always
            :attr:`~telegram.BackgroundFill.FREEFORM_GRADIENT`.
        colors (Sequence[:obj:`int`]): A list of the 3 or 4 base colors that are used to
            generate the freeform gradient in the `RGB24` format
    """

    __slots__ = ("colors",)

    def __init__(
        self,
        colors: Sequence[int],
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(type=self.FREEFORM_GRADIENT, api_kwargs=api_kwargs)

        with self._unfrozen():
            self.colors: tuple[int, ...] = parse_sequence_arg(colors)

            self._id_attrs = (self.colors,)


class BackgroundType(TelegramObject):
    """Base class for Telegram BackgroundType Objects. It can be one of:

    * :class:`telegram.BackgroundTypeFill`
    * :class:`telegram.BackgroundTypeWallpaper`
    * :class:`telegram.BackgroundTypePattern`
    * :class:`telegram.BackgroundTypeChatTheme`.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`type` is equal.

    .. versionadded:: 21.2

    Args:
        type (:obj:`str`): Type of the background. Can be one of:
            :attr:`~telegram.BackgroundType.FILL`, :attr:`~telegram.BackgroundType.WALLPAPER`
            :attr:`~telegram.BackgroundType.PATTERN` or
            :attr:`~telegram.BackgroundType.CHAT_THEME`.

    Attributes:
        type (:obj:`str`): Type of the background. Can be one of:
            :attr:`~telegram.BackgroundType.FILL`, :attr:`~telegram.BackgroundType.WALLPAPER`
            :attr:`~telegram.BackgroundType.PATTERN` or
            :attr:`~telegram.BackgroundType.CHAT_THEME`.

    """

    __slots__ = ("type",)

    FILL: Final[constants.BackgroundTypeType] = constants.BackgroundTypeType.FILL
    """:const:`telegram.constants.BackgroundTypeType.FILL`"""
    WALLPAPER: Final[constants.BackgroundTypeType] = constants.BackgroundTypeType.WALLPAPER
    """:const:`telegram.constants.BackgroundTypeType.WALLPAPER`"""
    PATTERN: Final[constants.BackgroundTypeType] = constants.BackgroundTypeType.PATTERN
    """:const:`telegram.constants.BackgroundTypeType.PATTERN`"""
    CHAT_THEME: Final[constants.BackgroundTypeType] = constants.BackgroundTypeType.CHAT_THEME
    """:const:`telegram.constants.BackgroundTypeType.CHAT_THEME`"""

    def __init__(
        self,
        type: str,  # pylint: disable=redefined-builtin
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required by all subclasses
        self.type: str = enum.get_member(constants.BackgroundTypeType, type, type)

        self._id_attrs = (self.type,)
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "BackgroundType":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        _class_mapping: dict[str, type[BackgroundType]] = {
            cls.FILL: BackgroundTypeFill,
            cls.WALLPAPER: BackgroundTypeWallpaper,
            cls.PATTERN: BackgroundTypePattern,
            cls.CHAT_THEME: BackgroundTypeChatTheme,
        }

        if cls is BackgroundType and data.get("type") in _class_mapping:
            return _class_mapping[data.pop("type")].de_json(data=data, bot=bot)

        if "fill" in data:
            data["fill"] = de_json_optional(data.get("fill"), BackgroundFill, bot)

        if "document" in data:
            data["document"] = de_json_optional(data.get("document"), Document, bot)

        return super().de_json(data=data, bot=bot)


class BackgroundTypeFill(BackgroundType):
    """
    The background is automatically filled based on the selected colors.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`fill` and :attr:`dark_theme_dimming` are equal.

    .. versionadded:: 21.2

    Args:
        fill (:class:`telegram.BackgroundFill`): The background fill.
        dark_theme_dimming (:obj:`int`): Dimming of the background in dark themes, as a
            percentage;
            0-:tg-const:`telegram.constants.BackgroundTypeLimit.MAX_DIMMING`.

    Attributes:
        type (:obj:`str`): Type of the background. Always
            :attr:`~telegram.BackgroundType.FILL`.
        fill (:class:`telegram.BackgroundFill`): The background fill.
        dark_theme_dimming (:obj:`int`): Dimming of the background in dark themes, as a
            percentage;
            0-:tg-const:`telegram.constants.BackgroundTypeLimit.MAX_DIMMING`.
    """

    __slots__ = ("dark_theme_dimming", "fill")

    def __init__(
        self,
        fill: BackgroundFill,
        dark_theme_dimming: int,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(type=self.FILL, api_kwargs=api_kwargs)

        with self._unfrozen():
            self.fill: BackgroundFill = fill
            self.dark_theme_dimming: int = dark_theme_dimming

            self._id_attrs = (self.fill, self.dark_theme_dimming)


class BackgroundTypeWallpaper(BackgroundType):
    """
    The background is a wallpaper in the `JPEG` format.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`document` and :attr:`dark_theme_dimming` are equal.

    .. versionadded:: 21.2

    Args:
        document (:class:`telegram.Document`): Document with the wallpaper
        dark_theme_dimming (:obj:`int`): Dimming of the background in dark themes, as a
            percentage;
            0-:tg-const:`telegram.constants.BackgroundTypeLimit.MAX_DIMMING`.
        is_blurred (:obj:`bool`, optional): :obj:`True`, if the wallpaper is downscaled to fit
            in a 450x450 square and then box-blurred with radius 12
        is_moving (:obj:`bool`, optional): :obj:`True`, if the background moves slightly
            when the device is tilted

    Attributes:
        type (:obj:`str`): Type of the background. Always
            :attr:`~telegram.BackgroundType.WALLPAPER`.
        document (:class:`telegram.Document`): Document with the wallpaper
        dark_theme_dimming (:obj:`int`): Dimming of the background in dark themes, as a
            percentage;
            0-:tg-const:`telegram.constants.BackgroundTypeLimit.MAX_DIMMING`.
        is_blurred (:obj:`bool`): Optional. :obj:`True`, if the wallpaper is downscaled to fit
            in a 450x450 square and then box-blurred with radius 12
        is_moving (:obj:`bool`): Optional. :obj:`True`, if the background moves slightly
            when the device is tilted
    """

    __slots__ = ("dark_theme_dimming", "document", "is_blurred", "is_moving")

    def __init__(
        self,
        document: Document,
        dark_theme_dimming: int,
        is_blurred: bool | None = None,
        is_moving: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(type=self.WALLPAPER, api_kwargs=api_kwargs)

        with self._unfrozen():
            # Required
            self.document: Document = document
            self.dark_theme_dimming: int = dark_theme_dimming
            # Optionals
            self.is_blurred: bool | None = is_blurred
            self.is_moving: bool | None = is_moving

            self._id_attrs = (self.document, self.dark_theme_dimming)


class BackgroundTypePattern(BackgroundType):
    """
    The background is a ``.PNG`` or ``.TGV`` (gzipped subset of ``SVG`` with ``MIME`` type
    ``"application/x-tgwallpattern"``) pattern to be combined with the background fill
    chosen by the user.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`document` and :attr:`fill` and :attr:`intensity` are equal.

    .. versionadded:: 21.2

    Args:
        document (:class:`telegram.Document`): Document with the pattern.
        fill (:class:`telegram.BackgroundFill`): The background fill that is combined with
            the pattern.
        intensity (:obj:`int`): Intensity of the pattern when it is shown above the filled
            background;
            0-:tg-const:`telegram.constants.BackgroundTypeLimit.MAX_INTENSITY`.
        is_inverted (:obj:`int`, optional): :obj:`True`, if the background fill must be applied
            only to the pattern itself. All other pixels are black in this case. For dark
            themes only.
        is_moving (:obj:`bool`, optional): :obj:`True`, if the background moves slightly
            when the device is tilted.

    Attributes:
        type (:obj:`str`): Type of the background. Always
            :attr:`~telegram.BackgroundType.PATTERN`.
        document (:class:`telegram.Document`): Document with the pattern.
        fill (:class:`telegram.BackgroundFill`): The background fill that is combined with
            the pattern.
        intensity (:obj:`int`): Intensity of the pattern when it is shown above the filled
            background;
            0-:tg-const:`telegram.constants.BackgroundTypeLimit.MAX_INTENSITY`.
        is_inverted (:obj:`int`): Optional. :obj:`True`, if the background fill must be applied
            only to the pattern itself. All other pixels are black in this case. For dark
            themes only.
        is_moving (:obj:`bool`): Optional. :obj:`True`, if the background moves slightly
            when the device is tilted.
    """

    __slots__ = (
        "document",
        "fill",
        "intensity",
        "is_inverted",
        "is_moving",
    )

    def __init__(
        self,
        document: Document,
        fill: BackgroundFill,
        intensity: int,
        is_inverted: bool | None = None,
        is_moving: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(type=self.PATTERN, api_kwargs=api_kwargs)

        with self._unfrozen():
            # Required
            self.document: Document = document
            self.fill: BackgroundFill = fill
            self.intensity: int = intensity
            # Optionals
            self.is_inverted: bool | None = is_inverted
            self.is_moving: bool | None = is_moving

            self._id_attrs = (self.document, self.fill, self.intensity)


class BackgroundTypeChatTheme(BackgroundType):
    """
    The background is taken directly from a built-in chat theme.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`theme_name` is equal.

    .. versionadded:: 21.2

    Args:
        theme_name (:obj:`str`): Name of the chat theme, which is usually an emoji.

    Attributes:
        type (:obj:`str`): Type of the background. Always
            :attr:`~telegram.BackgroundType.CHAT_THEME`.
        theme_name (:obj:`str`): Name of the chat theme, which is usually an emoji.
    """

    __slots__ = ("theme_name",)

    def __init__(
        self,
        theme_name: str,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(type=self.CHAT_THEME, api_kwargs=api_kwargs)

        with self._unfrozen():
            self.theme_name: str = theme_name

            self._id_attrs = (self.theme_name,)


class ChatBackground(TelegramObject):
    """
    This object represents a chat background.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`type` is  equal.

    .. versionadded:: 21.2

    Args:
        type (:class:`telegram.BackgroundType`): Type of the background.

    Attributes:
        type (:class:`telegram.BackgroundType`): Type of the background.
    """

    __slots__ = ("type",)

    def __init__(
        self,
        type: BackgroundType,  # pylint: disable=redefined-builtin
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.type: BackgroundType = type

        self._id_attrs = (self.type,)
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "ChatBackground":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["type"] = de_json_optional(data.get("type"), BackgroundType, bot)

        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import datetime as dtm
import inspect

import pytest

from telegram import User
from telegram.ext import Defaults
from telegram.warnings import PTBDeprecationWarning
from tests.auxil.envvars import TEST_WITH_OPT_DEPS
from tests.auxil.slots import mro_slots


class TestDefaults:
    def test_slot_behaviour(self):
        a = Defaults(parse_mode="HTML", do_quote=True)
        for attr in a.__slots__:
            assert getattr(a, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(a)) == len(set(mro_slots(a))), "duplicate slot"

    def test_utc(self):
        defaults = Defaults()
        assert defaults.tzinfo is dtm.timezone.utc

    @pytest.mark.skipif(not TEST_WITH_OPT_DEPS, reason="pytz not installed")
    def test_pytz_deprecation(self, recwarn):
        import pytz  # noqa: PLC0415

        with pytest.warns(PTBDeprecationWarning, match="pytz") as record:
            Defaults(tzinfo=pytz.timezone("Europe/Berlin"))

        assert record[0].category == PTBDeprecationWarning
        assert record[0].filename == __file__, "wrong stacklevel!"

    def test_data_assignment(self):
        defaults = Defaults()

        for name, _val in inspect.getmembers(Defaults, lambda x: isinstance(x, property)):
            with pytest.raises(AttributeError):
                setattr(defaults, name, True)

    def test_equality(self):
        a = Defaults(parse_mode="HTML", do_quote=True)
        b = Defaults(parse_mode="HTML", do_quote=True)
        c = Defaults(parse_mode="HTML", do_quote=True, protect_content=True)
        d = Defaults(parse_mode="HTML", protect_content=True)
        e = User(123, "test_user", False)
        f = Defaults(parse_mode="HTML", block=True)
        g = Defaults(parse_mode="HTML", block=True)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

        assert f == g
        assert hash(f) == hash(g)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import re
from pathlib import Path

from telegram._utils.strings import TextEncoding

telegram_root = Path(__file__).parent.parent / "telegram"
telegram_ext_root = telegram_root / "ext"
exclude_dirs = {
    # We touch passport stuff only if strictly necessary.
    telegram_root / "_passport",
}

exclude_patterns = {
    re.compile(re.escape("self.type: ReactionType = type")),
    re.compile(re.escape("self.type: BackgroundType = type")),
    re.compile(re.escape("self.type: StoryAreaType = type")),
}


def test_types_are_converted_to_enum():
    """We want to convert all attributes of name "type" to an enum from telegram.constants.
    Since we don't necessarily document this as type hint, we simply check this with a regex.
    """
    pattern = re.compile(r"self\.type: [^=]+ = ([^\n]+)\n", re.MULTILINE)

    for path in telegram_root.rglob("*.py"):
        if telegram_ext_root in path.parents or any(
            exclude_dir in path.parents for exclude_dir in exclude_dirs
        ):
            # We don't check tg.ext.
            continue

        text = path.read_text(encoding=TextEncoding.UTF_8)
        for match in re.finditer(pattern, text):
            if any(exclude_pattern.match(match.group(0)) for exclude_pattern in exclude_patterns):
                continue

            assert match.group(1).startswith("enum.get_member") or match.group(1).startswith(
                "get_member"
            ), (
                f"`{match.group(1)}` in `{path}` does not seem to convert the type to an enum. "
                f"Please fix this and also make sure to add a separate test to the classes test "
                f"file."
            )



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import (
    Dice,
    MenuButton,
    MenuButtonCommands,
    MenuButtonDefault,
    MenuButtonWebApp,
    WebAppInfo,
)
from telegram.constants import MenuButtonType
from tests.auxil.slots import mro_slots


@pytest.fixture
def menu_button():
    return MenuButton(MenuButtonTestBase.type)


class MenuButtonTestBase:
    type = MenuButtonType.DEFAULT
    text = "this is a test string"
    web_app = WebAppInfo(url="https://python-telegram-bot.org")


class TestMenuButtonWithoutRequest(MenuButtonTestBase):
    def test_slot_behaviour(self, menu_button):
        inst = menu_button
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_type_enum_conversion(self, menu_button):
        assert type(MenuButton("default").type) is MenuButtonType
        assert MenuButton("unknown").type == "unknown"

    def test_de_json(self, offline_bot):
        data = {"type": "unknown"}
        transaction_partner = MenuButton.de_json(data, offline_bot)
        assert transaction_partner.api_kwargs == {}
        assert transaction_partner.type == "unknown"

    @pytest.mark.parametrize(
        ("mb_type", "subclass"),
        [
            ("commands", MenuButtonCommands),
            ("web_app", MenuButtonWebApp),
            ("default", MenuButtonDefault),
        ],
    )
    def test_de_json_subclass(self, offline_bot, mb_type, subclass):
        json_dict = {
            "type": mb_type,
            "web_app": self.web_app.to_dict(),
            "text": self.text,
        }
        mb = MenuButton.de_json(json_dict, offline_bot)

        assert type(mb) is subclass
        assert set(mb.api_kwargs.keys()) == set(json_dict.keys()) - set(subclass.__slots__) - {
            "type"
        }
        assert mb.type == mb_type

    def test_to_dict(self, menu_button):
        assert menu_button.to_dict() == {"type": menu_button.type}

    def test_equality(self, menu_button):
        a = menu_button
        b = MenuButton(self.type)
        c = MenuButton("unknown")
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def menu_button_commands():
    return MenuButtonCommands()


class TestMenuButtonCommandsWithoutRequest(MenuButtonTestBase):
    type = MenuButtonType.COMMANDS

    def test_slot_behaviour(self, menu_button_commands):
        inst = menu_button_commands
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        transaction_partner = MenuButtonCommands.de_json({}, offline_bot)
        assert transaction_partner.api_kwargs == {}
        assert transaction_partner.type == "commands"

    def test_to_dict(self, menu_button_commands):
        assert menu_button_commands.to_dict() == {"type": menu_button_commands.type}

    def test_equality(self, menu_button_commands):
        a = menu_button_commands
        b = MenuButtonCommands()
        c = Dice(5, "test")
        d = MenuButtonDefault()

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def menu_button_default():
    return MenuButtonDefault()


class TestMenuButtonDefaultWithoutRequest(MenuButtonTestBase):
    type = MenuButtonType.DEFAULT

    def test_slot_behaviour(self, menu_button_default):
        inst = menu_button_default
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        transaction_partner = MenuButtonDefault.de_json({}, offline_bot)
        assert transaction_partner.api_kwargs == {}
        assert transaction_partner.type == "default"

    def test_to_dict(self, menu_button_default):
        assert menu_button_default.to_dict() == {"type": menu_button_default.type}

    def test_equality(self, menu_button_default):
        a = menu_button_default
        b = MenuButtonDefault()
        c = Dice(5, "test")
        d = MenuButtonCommands()

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def menu_button_web_app():
    return MenuButtonWebApp(
        web_app=TestMenuButtonWebAppWithoutRequest.web_app,
        text=TestMenuButtonWebAppWithoutRequest.text,
    )


class TestMenuButtonWebAppWithoutRequest(MenuButtonTestBase):
    type = MenuButtonType.WEB_APP

    def test_slot_behaviour(self, menu_button_web_app):
        inst = menu_button_web_app
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {"web_app": self.web_app.to_dict(), "text": self.text}
        transaction_partner = MenuButtonWebApp.de_json(json_dict, offline_bot)
        assert transaction_partner.api_kwargs == {}
        assert transaction_partner.type == "web_app"
        assert transaction_partner.web_app == self.web_app
        assert transaction_partner.text == self.text

    def test_to_dict(self, menu_button_web_app):
        assert menu_button_web_app.to_dict() == {
            "type": menu_button_web_app.type,
            "web_app": menu_button_web_app.web_app.to_dict(),
            "text": menu_button_web_app.text,
        }

    def test_equality(self, menu_button_web_app):
        a = menu_button_web_app
        b = MenuButtonWebApp(web_app=self.web_app, text=self.text)
        c = MenuButtonWebApp(web_app=self.web_app, text="other text")
        d = MenuButtonWebApp(web_app=WebAppInfo(url="https://example.org"), text=self.text)
        e = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import json
from typing import Any
from urllib.parse import quote

import pytest

from telegram import InputFile, InputMediaPhoto, InputMediaVideo, MessageEntity
from telegram.request import RequestData
from telegram.request._requestparameter import RequestParameter
from tests.auxil.files import data_file
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def inputfiles() -> dict[bool, InputFile]:
    return {True: InputFile(obj="data", attach=True), False: InputFile(obj="data", attach=False)}


@pytest.fixture(scope="module")
def input_media_video() -> InputMediaVideo:
    return InputMediaVideo(
        media=data_file("telegram.mp4").read_bytes(),
        thumbnail=data_file("telegram.jpg").read_bytes(),
        parse_mode=None,
    )


@pytest.fixture(scope="module")
def input_media_photo() -> InputMediaPhoto:
    return InputMediaPhoto(
        media=data_file("telegram.jpg").read_bytes(),
        parse_mode=None,
    )


@pytest.fixture(scope="module")
def simple_params() -> dict[str, Any]:
    return {
        "string": "string",
        "integer": 1,
        "tg_object": MessageEntity("type", 1, 1),
        "list": [1, "string", MessageEntity("type", 1, 1)],
    }


@pytest.fixture(scope="module")
def simple_jsons() -> dict[str, Any]:
    return {
        "string": "string",
        "integer": json.dumps(1),
        "tg_object": MessageEntity("type", 1, 1).to_json(),
        "list": json.dumps([1, "string", MessageEntity("type", 1, 1).to_dict()]),
    }


@pytest.fixture(scope="module")
def simple_rqs(simple_params) -> RequestData:
    return RequestData(
        [RequestParameter.from_input(key, value) for key, value in simple_params.items()]
    )


@pytest.fixture(scope="module")
def file_params(inputfiles, input_media_video, input_media_photo) -> dict[str, Any]:
    return {
        "inputfile_attach": inputfiles[True],
        "inputfile_no_attach": inputfiles[False],
        "inputmedia": input_media_video,
        "inputmedia_list": [input_media_video, input_media_photo],
    }


@pytest.fixture(scope="module")
def file_jsons(inputfiles, input_media_video, input_media_photo) -> dict[str, Any]:
    input_media_video_dict = input_media_video.to_dict()
    input_media_video_dict["media"] = input_media_video.media.attach_uri
    input_media_video_dict["thumbnail"] = input_media_video.thumbnail.attach_uri
    input_media_photo_dict = input_media_photo.to_dict()
    input_media_photo_dict["media"] = input_media_photo.media.attach_uri
    return {
        "inputfile_attach": inputfiles[True].attach_uri,
        "inputmedia": json.dumps(input_media_video_dict),
        "inputmedia_list": json.dumps([input_media_video_dict, input_media_photo_dict]),
    }


@pytest.fixture(scope="module")
def file_rqs(file_params) -> RequestData:
    return RequestData(
        [RequestParameter.from_input(key, value) for key, value in file_params.items()]
    )


@pytest.fixture(scope="module")
def mixed_params(file_params, simple_params) -> dict[str, Any]:
    both = file_params.copy()
    both.update(simple_params)
    return both


@pytest.fixture(scope="module")
def mixed_jsons(file_jsons, simple_jsons) -> dict[str, Any]:
    both = file_jsons.copy()
    both.update(simple_jsons)
    return both


@pytest.fixture(scope="module")
def mixed_rqs(mixed_params) -> RequestData:
    return RequestData(
        [RequestParameter.from_input(key, value) for key, value in mixed_params.items()]
    )


class TestRequestDataWithoutRequest:
    def test_slot_behaviour(self, simple_rqs):
        for attr in simple_rqs.__slots__:
            assert getattr(simple_rqs, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(simple_rqs)) == len(set(mro_slots(simple_rqs))), "duplicate slot"

    def test_contains_files(self, simple_rqs, file_rqs, mixed_rqs):
        assert not simple_rqs.contains_files
        assert file_rqs.contains_files
        assert mixed_rqs.contains_files

    def test_parameters(
        self, simple_rqs, file_rqs, mixed_rqs, inputfiles, input_media_video, input_media_photo
    ):
        simple_params_expected = {
            "string": "string",
            "integer": 1,
            "tg_object": MessageEntity("type", 1, 1).to_dict(),
            "list": [1, "string", MessageEntity("type", 1, 1).to_dict()],
        }
        video_value = {
            "media": input_media_video.media.attach_uri,
            "thumbnail": input_media_video.thumbnail.attach_uri,
            "type": input_media_video.type,
        }
        photo_value = {"media": input_media_photo.media.attach_uri, "type": input_media_photo.type}
        file_params_expected = {
            "inputfile_attach": inputfiles[True].attach_uri,
            "inputmedia": video_value,
            "inputmedia_list": [video_value, photo_value],
        }
        mixed_params_expected = simple_params_expected.copy()
        mixed_params_expected.update(file_params_expected)

        assert simple_rqs.parameters == simple_params_expected
        assert file_rqs.parameters == file_params_expected
        assert mixed_rqs.parameters == mixed_params_expected

    def test_json_parameters(
        self, simple_rqs, file_rqs, mixed_rqs, simple_jsons, file_jsons, mixed_jsons
    ):
        assert simple_rqs.json_parameters == simple_jsons
        assert file_rqs.json_parameters == file_jsons
        assert mixed_rqs.json_parameters == mixed_jsons

    def test_json_payload(
        self, simple_rqs, file_rqs, mixed_rqs, simple_jsons, file_jsons, mixed_jsons
    ):
        assert simple_rqs.json_payload == json.dumps(simple_jsons).encode()
        assert file_rqs.json_payload == json.dumps(file_jsons).encode()
        assert mixed_rqs.json_payload == json.dumps(mixed_jsons).encode()

    def test_multipart_data(
        self,
        simple_rqs,
        file_rqs,
        mixed_rqs,
        inputfiles,
        input_media_video,
        input_media_photo,
    ):
        expected = {
            inputfiles[True].attach_name: inputfiles[True].field_tuple,
            "inputfile_no_attach": inputfiles[False].field_tuple,
            input_media_photo.media.attach_name: input_media_photo.media.field_tuple,
            input_media_video.media.attach_name: input_media_video.media.field_tuple,
            input_media_video.thumbnail.attach_name: input_media_video.thumbnail.field_tuple,
        }
        assert simple_rqs.multipart_data == {}
        assert file_rqs.multipart_data == expected
        assert mixed_rqs.multipart_data == expected

    def test_url_encoding(self):
        data = RequestData(
            [
                RequestParameter.from_input("chat_id", 123),
                RequestParameter.from_input("text", "Hello there/!"),
            ]
        )
        expected_params = "chat_id=123&text=Hello+there%2F%21"
        expected_url = "https://te.st/method?" + expected_params
        assert data.url_encoded_parameters() == expected_params
        assert data.parametrized_url("https://te.st/method") == expected_url

        expected_params = "chat_id=123&text=Hello%20there/!"
        expected_url = "https://te.st/method?" + expected_params
        assert (
            data.url_encoded_parameters(encode_kwargs={"quote_via": quote, "safe": "/!"})
            == expected_params
        )
        assert (
            data.parametrized_url(
                "https://te.st/method", encode_kwargs={"quote_via": quote, "safe": "/!"}
            )
            == expected_url
        )



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram MessageReaction Update."""

import datetime as dtm
from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._chat import Chat
from telegram._reaction import ReactionCount, ReactionType
from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.argumentparsing import de_json_optional, de_list_optional, parse_sequence_arg
from telegram._utils.datetime import extract_tzinfo_from_defaults, from_timestamp
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class MessageReactionCountUpdated(TelegramObject):
    """This class represents reaction changes on a message with anonymous reactions.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if the :attr:`chat`, :attr:`message_id`, :attr:`date` and :attr:`reactions`
    is equal.

    .. versionadded:: 20.8

    Args:
        chat (:class:`telegram.Chat`): The chat containing the message.
        message_id (:obj:`int`): Unique message identifier inside the chat.
        date (:class:`datetime.datetime`): Date of the change in Unix time
            |datetime_localization|
        reactions (Sequence[:class:`telegram.ReactionCount`]): List of reactions that are present
            on the message

    Attributes:
        chat (:class:`telegram.Chat`): The chat containing the message.
        message_id (:obj:`int`): Unique message identifier inside the chat.
        date (:class:`datetime.datetime`): Date of the change in Unix time
            |datetime_localization|
        reactions (tuple[:class:`telegram.ReactionCount`]): List of reactions that are present on
            the message
    """

    __slots__ = (
        "chat",
        "date",
        "message_id",
        "reactions",
    )

    def __init__(
        self,
        chat: Chat,
        message_id: int,
        date: dtm.datetime,
        reactions: Sequence[ReactionCount],
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.chat: Chat = chat
        self.message_id: int = message_id
        self.date: dtm.datetime = date
        self.reactions: tuple[ReactionCount, ...] = parse_sequence_arg(reactions)

        self._id_attrs = (self.chat, self.message_id, self.date, self.reactions)
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "MessageReactionCountUpdated":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)

        data["date"] = from_timestamp(data.get("date"), tzinfo=loc_tzinfo)
        data["chat"] = de_json_optional(data.get("chat"), Chat, bot)
        data["reactions"] = de_list_optional(data.get("reactions"), ReactionCount, bot)

        return super().de_json(data=data, bot=bot)


class MessageReactionUpdated(TelegramObject):
    """This class represents a change of a reaction on a message performed by a user.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if the :attr:`chat`, :attr:`message_id`, :attr:`date`, :attr:`old_reaction`
    and :attr:`new_reaction` is equal.

    .. versionadded:: 20.8

    Args:
        chat (:class:`telegram.Chat`): The chat containing the message.
        message_id (:obj:`int`): Unique message identifier inside the chat.
        date (:class:`datetime.datetime`): Date of the change in Unix time.
            |datetime_localization|
        old_reaction (Sequence[:class:`telegram.ReactionType`]): Previous list of reaction types
            that were set by the user.
        new_reaction (Sequence[:class:`telegram.ReactionType`]): New list of reaction types that
            were set by the user.
        user (:class:`telegram.User`, optional): The user that changed the reaction, if the user
            isn't anonymous.
        actor_chat (:class:`telegram.Chat`, optional): The chat on behalf of which the reaction was
            changed, if the user is anonymous.

    Attributes:
        chat (:class:`telegram.Chat`): The chat containing the message.
        message_id (:obj:`int`): Unique message identifier inside the chat.
        date (:class:`datetime.datetime`): Date of the change in Unix time.
            |datetime_localization|
        old_reaction (tuple[:class:`telegram.ReactionType`]): Previous list of reaction types
            that were set by the user.
        new_reaction (tuple[:class:`telegram.ReactionType`]): New list of reaction types that
            were set by the user.
        user (:class:`telegram.User`): Optional. The user that changed the reaction, if the user
            isn't anonymous.
        actor_chat (:class:`telegram.Chat`): Optional. The chat on behalf of which the reaction was
            changed, if the user is anonymous.
    """

    __slots__ = (
        "actor_chat",
        "chat",
        "date",
        "message_id",
        "new_reaction",
        "old_reaction",
        "user",
    )

    def __init__(
        self,
        chat: Chat,
        message_id: int,
        date: dtm.datetime,
        old_reaction: Sequence[ReactionType],
        new_reaction: Sequence[ReactionType],
        user: User | None = None,
        actor_chat: Chat | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.chat: Chat = chat
        self.message_id: int = message_id
        self.date: dtm.datetime = date
        self.old_reaction: tuple[ReactionType, ...] = parse_sequence_arg(old_reaction)
        self.new_reaction: tuple[ReactionType, ...] = parse_sequence_arg(new_reaction)

        # Optional
        self.user: User | None = user
        self.actor_chat: Chat | None = actor_chat

        self._id_attrs = (
            self.chat,
            self.message_id,
            self.date,
            self.old_reaction,
            self.new_reaction,
        )
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "MessageReactionUpdated":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)

        data["date"] = from_timestamp(data.get("date"), tzinfo=loc_tzinfo)
        data["chat"] = de_json_optional(data.get("chat"), Chat, bot)
        data["old_reaction"] = de_list_optional(data.get("old_reaction"), ReactionType, bot)
        data["new_reaction"] = de_list_optional(data.get("new_reaction"), ReactionType, bot)
        data["user"] = de_json_optional(data.get("user"), User, bot)
        data["actor_chat"] = de_json_optional(data.get("actor_chat"), Chat, bot)

        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program. If not, see [http://www.gnu.org/licenses/].
# pylint: disable=redefined-builtin
"""This module contains the classes for Telegram Stars Revenue Withdrawals."""

import datetime as dtm
from typing import TYPE_CHECKING, Final

from telegram import constants
from telegram._telegramobject import TelegramObject
from telegram._utils import enum
from telegram._utils.datetime import extract_tzinfo_from_defaults, from_timestamp
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class RevenueWithdrawalState(TelegramObject):
    """This object describes the state of a revenue withdrawal operation. Currently, it can be one
    of:

    * :class:`telegram.RevenueWithdrawalStatePending`
    * :class:`telegram.RevenueWithdrawalStateSucceeded`
    * :class:`telegram.RevenueWithdrawalStateFailed`

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`type` is equal.

    .. versionadded:: 21.4

    Args:
        type (:obj:`str`): The type of the state.

    Attributes:
        type (:obj:`str`): The type of the state.
    """

    __slots__ = ("type",)

    PENDING: Final[str] = constants.RevenueWithdrawalStateType.PENDING
    """:const:`telegram.constants.RevenueWithdrawalStateType.PENDING`"""
    SUCCEEDED: Final[str] = constants.RevenueWithdrawalStateType.SUCCEEDED
    """:const:`telegram.constants.RevenueWithdrawalStateType.SUCCEEDED`"""
    FAILED: Final[str] = constants.RevenueWithdrawalStateType.FAILED
    """:const:`telegram.constants.RevenueWithdrawalStateType.FAILED`"""

    def __init__(self, type: str, *, api_kwargs: JSONDict | None = None) -> None:
        super().__init__(api_kwargs=api_kwargs)
        self.type: str = enum.get_member(constants.RevenueWithdrawalStateType, type, type)

        self._id_attrs = (self.type,)
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "RevenueWithdrawalState":
        """Converts JSON data to the appropriate :class:`RevenueWithdrawalState` object, i.e. takes
        care of selecting the correct subclass.

        Args:
            data (dict[:obj:`str`, ...]): The JSON data.
            bot (:class:`telegram.Bot`): The bot associated with this object.

        Returns:
            The Telegram object.

        """
        data = cls._parse_data(data)

        _class_mapping: dict[str, type[RevenueWithdrawalState]] = {
            cls.PENDING: RevenueWithdrawalStatePending,
            cls.SUCCEEDED: RevenueWithdrawalStateSucceeded,
            cls.FAILED: RevenueWithdrawalStateFailed,
        }

        if cls is RevenueWithdrawalState and data.get("type") in _class_mapping:
            return _class_mapping[data.pop("type")].de_json(data=data, bot=bot)

        return super().de_json(data=data, bot=bot)


class RevenueWithdrawalStatePending(RevenueWithdrawalState):
    """The withdrawal is in progress.

    .. versionadded:: 21.4

    Attributes:
        type (:obj:`str`): The type of the state, always
            :tg-const:`telegram.RevenueWithdrawalState.PENDING`.
    """

    __slots__ = ()

    def __init__(self, *, api_kwargs: JSONDict | None = None) -> None:
        super().__init__(type=RevenueWithdrawalState.PENDING, api_kwargs=api_kwargs)
        self._freeze()


class RevenueWithdrawalStateSucceeded(RevenueWithdrawalState):
    """The withdrawal succeeded.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`date` are equal.

    .. versionadded:: 21.4

    Args:
        date (:obj:`datetime.datetime`): Date the withdrawal was completed as a datetime object.
        url (:obj:`str`): An HTTPS URL that can be used to see transaction details.

    Attributes:
        type (:obj:`str`): The type of the state, always
            :tg-const:`telegram.RevenueWithdrawalState.SUCCEEDED`.
        date (:obj:`datetime.datetime`): Date the withdrawal was completed as a datetime object.
        url (:obj:`str`): An HTTPS URL that can be used to see transaction details.
    """

    __slots__ = ("date", "url")

    def __init__(
        self,
        date: dtm.datetime,
        url: str,
        *,
        api_kwargs: JSONDict | None = None,
    ) -> None:
        super().__init__(type=RevenueWithdrawalState.SUCCEEDED, api_kwargs=api_kwargs)

        with self._unfrozen():
            self.date: dtm.datetime = date
            self.url: str = url
            self._id_attrs = (
                self.type,
                self.date,
            )

    @classmethod
    def de_json(
        cls, data: JSONDict, bot: "Bot | None" = None
    ) -> "RevenueWithdrawalStateSucceeded":
        """See :meth:`telegram.RevenueWithdrawalState.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)
        data["date"] = from_timestamp(data.get("date", None), tzinfo=loc_tzinfo)

        return super().de_json(data=data, bot=bot)  # type: ignore[return-value]


class RevenueWithdrawalStateFailed(RevenueWithdrawalState):
    """The withdrawal failed and the transaction was refunded.

    .. versionadded:: 21.4

    Attributes:
        type (:obj:`str`): The type of the state, always
            :tg-const:`telegram.RevenueWithdrawalState.FAILED`.
    """

    __slots__ = ()

    def __init__(self, *, api_kwargs: JSONDict | None = None) -> None:
        super().__init__(type=RevenueWithdrawalState.FAILED, api_kwargs=api_kwargs)
        self._freeze()
